@@ -76,6 +76,12 @@
 
 #define MAXNAME (DNS_NAME_MAXTEXT + 1)
 
+/*
+ * Default maximum number of chained queries before we give up
+ * to prevent CNAME loops.
+ */
+#define MAX_RESTARTS 11
+
 /* Variables used internally by delv. */
 char *progname;
 static isc_mem_t *mctx = NULL;
@@ -205,10 +211,10 @@ usage(void) {
 		"process)\n"
 		"                 +[no]yaml           (Present the results as "
 		"YAML)\n");
-	exit(1);
+	exit(EXIT_FAILURE);
 }
 
-ISC_NORETURN static void
+noreturn static void
 fatal(const char *format, ...) ISC_FORMAT_PRINTF(1, 2);
 
 static void
@@ -221,7 +227,7 @@ fatal(const char *format, ...) {
 	vfprintf(stderr, format, args);
 	va_end(args);
 	fprintf(stderr, "\n");
-	exit(1);
+	_exit(EXIT_FAILURE);
 }
 
 static void
@@ -458,7 +464,8 @@ printdata(dns_rdataset_t *rdataset, dns_name_t *owner,
 			     result = dns_rdataset_next(rdataset))
 			{
 				if ((rdataset->attributes &
-				     DNS_RDATASETATTR_NEGATIVE) != 0) {
+				     DNS_RDATASETATTR_NEGATIVE) != 0)
+				{
 					continue;
 				}
 
@@ -482,7 +489,8 @@ printdata(dns_rdataset_t *rdataset, dns_name_t *owner,
 		} else {
 			dns_indent_t indent = { "  ", 2 };
 			if (!yaml && (rdataset->attributes &
-				      DNS_RDATASETATTR_NEGATIVE) != 0) {
+				      DNS_RDATASETATTR_NEGATIVE) != 0)
+			{
 				isc_buffer_putstr(&target, "; ");
 			}
 			result = dns_master_rdatasettotext(
@@ -784,7 +792,8 @@ load_keys(const cfg_obj_t *keys, dns_client_t *client) {
 		keylist = cfg_listelt_value(elt);
 
 		for (elt2 = cfg_list_first(keylist); elt2 != NULL;
-		     elt2 = cfg_list_next(elt2)) {
+		     elt2 = cfg_list_next(elt2))
+		{
 			key = cfg_listelt_value(elt2);
 			CHECK(key_fromconfig(key, client));
 		}
@@ -845,6 +854,7 @@ setup_dnsseckeys(dns_client_t *client) {
 
 		isc_buffer_init(&b, anchortext, sizeof(anchortext) - 1);
 		isc_buffer_add(&b, sizeof(anchortext) - 1);
+		cfg_parser_reset(parser);
 		result = cfg_parse_buffer(parser, &b, NULL, 0,
 					  &cfg_type_bindkeys, 0, &bindkeys);
 		if (result != ISC_R_SUCCESS) {
@@ -942,7 +952,8 @@ addserver(dns_client_t *client) {
 		result = ISC_R_SUCCESS;
 		for (cur = res; cur != NULL; cur = cur->ai_next) {
 			if (cur->ai_family != AF_INET &&
-			    cur->ai_family != AF_INET6) {
+			    cur->ai_family != AF_INET6)
+			{
 				continue;
 			}
 			sa = isc_mem_get(mctx, sizeof(*sa));
@@ -1127,7 +1138,7 @@ plus_option(char *option) {
 			if (state) {
 				fprintf(stderr, "Invalid option: "
 						"+dlv is obsolete\n");
-				exit(1);
+				exit(EXIT_FAILURE);
 			}
 			break;
 		case 'n': /* dnssec */
@@ -1324,8 +1335,7 @@ dash_option(char *option, char *next, bool *open_type_class) {
 			break;
 		case 'h':
 			usage();
-			exit(0);
-		/* NOTREACHED */
+			exit(EXIT_SUCCESS);
 		case 'i':
 			no_sigs = true;
 			root_validation = false;
@@ -1335,11 +1345,9 @@ dash_option(char *option, char *next, bool *open_type_class) {
 			break;
 		case 'v':
 			fprintf(stderr, "delv %s\n", PACKAGE_VERSION);
-			exit(0);
-		/* NOTREACHED */
+			exit(EXIT_SUCCESS);
 		default:
-			INSIST(0);
-			ISC_UNREACHABLE();
+			UNREACHABLE();
 		}
 		if (strlen(option) > 1U) {
 			option = &option[1];
@@ -1445,7 +1453,8 @@ dash_option(char *option, char *next, bool *open_type_class) {
 				warn("extra query type");
 			}
 			if (rdtype == dns_rdatatype_ixfr ||
-			    rdtype == dns_rdatatype_axfr) {
+			    rdtype == dns_rdatatype_axfr)
+			{
 				fatal("Transfer not supported");
 			}
 			qtype = rdtype;
@@ -1469,7 +1478,7 @@ dash_option(char *option, char *next, bool *open_type_class) {
 			typeset = true;
 		} else {
 			fprintf(stderr, "Invalid IP address %s\n", value);
-			exit(1);
+			exit(EXIT_FAILURE);
 		}
 		return (value_from_next);
 	invalid_option:
@@ -1477,7 +1486,7 @@ dash_option(char *option, char *next, bool *open_type_class) {
 		fprintf(stderr, "Invalid option: -%s\n", option);
 		usage();
 	}
-	/* NOTREACHED */
+	UNREACHABLE();
 	return (false);
 }
 
@@ -1524,7 +1533,8 @@ preparse_args(int argc, char **argv) {
 
 		/* Look for dash value option. */
 		if (strpbrk(option, dash_opts) != &option[0] ||
-		    strlen(option) > 1U) {
+		    strlen(option) > 1U)
+		{
 			/* Error or value in option. */
 			continue;
 		}
@@ -1562,13 +1572,15 @@ parse_args(int argc, char **argv) {
 		} else if (argv[0][0] == '-') {
 			if (argc <= 1) {
 				if (dash_option(&argv[0][1], NULL,
-						&open_type_class)) {
+						&open_type_class))
+				{
 					argc--;
 					argv++;
 				}
 			} else {
 				if (dash_option(&argv[0][1], argv[1],
-						&open_type_class)) {
+						&open_type_class))
+				{
 					argc--;
 					argv++;
 				}
@@ -1587,7 +1599,8 @@ parse_args(int argc, char **argv) {
 						warn("extra query type");
 					}
 					if (rdtype == dns_rdatatype_ixfr ||
-					    rdtype == dns_rdatatype_axfr) {
+					    rdtype == dns_rdatatype_axfr)
+					{
 						fatal("Transfer not supported");
 					}
 					qtype = rdtype;
@@ -1769,6 +1782,8 @@ main(int argc, char *argv[]) {
 		goto cleanup;
 	}
 
+	dns_client_setmaxrestarts(client, MAX_RESTARTS);
+
 	/* Set the nameserver */
 	if (server != NULL) {
 		addserver(client);
