@@ -11,8 +11,7 @@
  * information regarding copyright ownership.
  */
 
-#ifndef NAMED_GLOBALS_H
-#define NAMED_GLOBALS_H 1
+#pragma once
 
 /*! \file */
 
@@ -62,19 +61,23 @@ EXTERN bool named_g_run_done INIT(false);
  *         for really short timers, another for client timers, and one
  *         for zone timers.
  */
-EXTERN isc_timermgr_t *named_g_timermgr	  INIT(NULL);
-EXTERN isc_socketmgr_t *named_g_socketmgr INIT(NULL);
-EXTERN isc_nm_t *named_g_nm		  INIT(NULL);
-EXTERN cfg_parser_t *named_g_parser	  INIT(NULL);
-EXTERN cfg_parser_t *named_g_addparser	  INIT(NULL);
-EXTERN const char *named_g_version	  INIT(VERSION);
-EXTERN const char *named_g_product	  INIT(PRODUCT);
-EXTERN const char *named_g_description	  INIT(DESCRIPTION);
-EXTERN const char *named_g_srcid	  INIT(SRCID);
-EXTERN const char *named_g_configargs	  INIT(CONFIGARGS);
-EXTERN const char *named_g_builder	  INIT(BUILDER);
-EXTERN in_port_t named_g_port		  INIT(0);
-EXTERN isc_dscp_t named_g_dscp		  INIT(-1);
+EXTERN isc_timermgr_t *named_g_timermgr INIT(NULL);
+EXTERN isc_nm_t *named_g_netmgr		INIT(NULL);
+EXTERN cfg_parser_t *named_g_parser	INIT(NULL);
+EXTERN cfg_parser_t *named_g_addparser	INIT(NULL);
+EXTERN const char *named_g_version	INIT(PACKAGE_VERSION);
+EXTERN const char *named_g_product	INIT(PACKAGE_NAME);
+EXTERN const char *named_g_description	INIT(PACKAGE_DESCRIPTION);
+EXTERN const char *named_g_srcid	INIT(PACKAGE_SRCID);
+EXTERN const char *named_g_configargs	INIT(PACKAGE_CONFIGARGS);
+EXTERN const char *named_g_builder	INIT(PACKAGE_BUILDER);
+EXTERN in_port_t named_g_port		INIT(0);
+EXTERN in_port_t named_g_tlsport	INIT(0);
+EXTERN in_port_t named_g_httpsport	INIT(0);
+EXTERN in_port_t named_g_httpport	INIT(0);
+
+EXTERN in_port_t named_g_http_listener_clients INIT(0);
+EXTERN in_port_t named_g_http_streams_per_conn INIT(0);
 
 EXTERN named_server_t *named_g_server INIT(NULL);
 
@@ -158,5 +161,3 @@ EXTERN dns_acl_t *named_g_mapped INIT(NULL);
 
 #undef EXTERN
 #undef INIT
-
-#endif /* NAMED_GLOBALS_H */
