@@ -11,12 +11,10 @@
 # See the COPYRIGHT file distributed with this work for additional
 # information regarding copyright ownership.
 
-rm -f supported
 rm -f */K* */dsset-* */*.signed */trusted.conf
 rm -f ns1/root.db
 rm -f ns1/signer.err
 rm -f ns2/good.db ns2/bad.db
-rm -f dig.out*
 rm -f */named.conf
 rm -f */named.run
 rm -f */named.memstats
