@@ -44,13 +44,11 @@
 
 #include <isc/commandline.h>
 #include <isc/hash.h>
-#include <isc/lib.h>
 #include <isc/mem.h>
 #include <isc/util.h>
 
 #include <dns/db.h>
 #include <dns/dyndb.h>
-#include <dns/lib.h>
 #include <dns/types.h>
 
 #include "db.h"
@@ -58,6 +56,8 @@
 #include "log.h"
 #include "util.h"
 
+/* aliases for the exported symbols */
+
 dns_dyndb_destroy_t dyndb_destroy;
 dns_dyndb_register_t dyndb_init;
 dns_dyndb_version_t dyndb_version;
@@ -95,21 +95,6 @@ dyndb_init(isc_mem_t *mctx, const char *name, const char *parameters,
 	REQUIRE(name != NULL);
 	REQUIRE(dctx != NULL);
 
-	/*
-	 * Depending on how dlopen() was called, we may not have
-	 * access to named's global namespace, in which case we need
-	 * to initialize libisc/libdns. We check this by comparing
-	 * the value of isc_mem_debugging to the value passed via
-	 * the context object.
-	 */
-	if (dctx->memdebug != &isc_mem_debugging) {
-		isc_lib_register();
-		isc_log_setcontext(dctx->lctx);
-		dns_log_setcontext(dctx->lctx);
-		isc_hash_set_initializer(dctx->hashinit);
-		isc_mem_debugging = *(unsigned int *)dctx->memdebug;
-	}
-
 	s = isc_mem_strdup(mctx, parameters);
 
 	result = isc_commandline_strtoargv(mctx, s, &argc, &argv, 0);
