@@ -26,12 +26,11 @@ echo_i "ns3/sign.sh"
 cp $infile $zonefile
 
 if [ -f ../ed448-supported.file ]; then
-	for i in Xexample.com.+016+09713 Xexample.com.+016+38353
-	do
-		cp "$i.key" "$(echo $i.key | sed s/X/K/)"
-		cp "$i.private" "$(echo $i.private | sed s/X/K/)"
-		cat "$(echo $i.key | sed s/X/K/)" >> "$zonefile"
-	done
+  for i in Xexample.com.+016+09713 Xexample.com.+016+38353; do
+    cp "$i.key" "$(echo $i.key | sed s/X/K/)"
+    cp "$i.private" "$(echo $i.private | sed s/X/K/)"
+    cat "$(echo $i.key | sed s/X/K/)" >>"$zonefile"
+  done
 fi
 
-$SIGNER -P -z -s "$starttime" -e "$endtime" -o "$zone" "$zonefile" > /dev/null 2> signer.err || cat signer.err
+$SIGNER -P -z -s "$starttime" -e "$endtime" -o "$zone" "$zonefile" >/dev/null 2>signer.err || cat signer.err
