@@ -9,10 +9,10 @@
 # See the COPYRIGHT file distributed with this work for additional
 # information regarding copyright ownership.
 
-import pytest_custom_markers
+import isctest.mark
 
 
 # The qmin test is inherently unstable, see GL #904 for details.
-@pytest_custom_markers.flaky(max_runs=3)
+@isctest.mark.flaky(max_runs=3)
 def test_qmin(run_tests_sh):
     run_tests_sh()
