@@ -15,32 +15,9 @@
 
 set -e
 
-SYSTEMTESTTOP=..
-. $SYSTEMTESTTOP/conf.sh
+. ../conf.sh
 
-USAGE="$0: [-DNx]"
-DEBUG=
-while getopts "DNx" c; do
-  case $c in
-    x)
-      set -x
-      DEBUG=-x
-      ;;
-    D) TEST_DNSRPS="-D" ;;
-    N) NOCLEAN=set ;;
-    *)
-      echo "$USAGE" 1>&2
-      exit 1
-      ;;
-  esac
-done
-shift $(expr $OPTIND - 1 || true)
-if test "$#" -ne 0; then
-  echo "$USAGE" 1>&2
-  exit 1
-fi
-
-[ ${NOCLEAN:-unset} = unset ] && $SHELL clean.sh $DEBUG
+$SHELL clean.sh
 
 $PERL testgen.pl
 
@@ -50,10 +27,11 @@ copy_setports ns2/named.conf.header.in ns2/named.conf.header
 copy_setports ns2/named.default.conf ns2/named.conf
 
 copy_setports ns3/named1.conf.in ns3/named.conf
-copy_setports ns3/named2.conf.in ns3/named2.conf
 
 copy_setports ns4/named.conf.in ns4/named.conf
 
+touch dnsrps.conf
+
 # setup policy zones for a 64-zone test
 i=1
 while test $i -le 64; do
@@ -65,24 +43,20 @@ while test $i -le 64; do
   j=1
   while test $j -le $i; do
     echo "name$j A 10.53.0.$i" >>ns2/db.max$i.local
-    j=$(expr $j + 1)
+    j=$((j + 1))
   done
-  i=$(expr $i + 1)
+  i=$((i + 1))
 done
 
-# decide whether to test DNSRPS
-$SHELL ../ckdnsrps.sh $TEST_DNSRPS $DEBUG
-test -z "$(grep 'dnsrps-enable yes' dnsrps.conf)" && TEST_DNSRPS=
-
 CWD=$(pwd)
 cat <<EOF >dnsrpzd.conf
 PID-FILE $CWD/dnsrpzd.pid;
 
 include $CWD/dnsrpzd-license-cur.conf
 
-zone "policy" { type master; file "$(pwd)/ns3/policy.db"; };
+zone "policy" { type primary; file "$(pwd)/ns3/policy.db"; };
 EOF
-sed -n -e 's/^ *//' -e "/zone.*.*master/s@file \"@&$CWD/ns2/@p" ns2/*.conf \
+sed -n -e 's/^ *//' -e "/zone.*.*primary/s@file \"@&$CWD/ns2/@p" ns2/*.conf \
   >>dnsrpzd.conf
 
 # Run dnsrpzd to get the license and prime the static policy zones
