@@ -11,11 +11,13 @@
 
 import concurrent.futures
 import os
-import subprocess
 import time
+
 import dns.query
 import dns.update
 
+import isctest
+
 
 def rndc_loop(test_state, server):
     rndc = os.getenv("RNDC")
@@ -33,11 +35,11 @@ def rndc_loop(test_state, server):
     ]
 
     while not test_state["finished"]:
-        subprocess.run(cmdline, check=False)
+        isctest.run.cmd(cmdline, raise_on_exception=False)
         time.sleep(1)
 
 
-def update_zone(test_state, zone, named_port, logger):
+def update_zone(test_state, zone, named_port):
     server = "10.53.0.2"
     for i in range(1000):
         if test_state["finished"]:
@@ -48,13 +50,13 @@ def update_zone(test_state, zone, named_port, logger):
             response = dns.query.udp(update, server, 10, named_port)
             assert response.rcode() == dns.rcode.NOERROR
         except dns.exception.Timeout:
-            logger.info(f"error: query timeout for {zone}")
+            isctest.log.info(f"error: query timeout for {zone}")
 
-    logger.info(f"Update of {server} zone {zone} successful")
+    isctest.log.info(f"Update of {server} zone {zone} successful")
 
 
 # If the test has run to completion without named crashing, it has succeeded.
-def test_update_stress(named_port, logger):
+def test_update_stress(named_port):
     test_state = {"finished": False}
 
     with concurrent.futures.ThreadPoolExecutor() as executor:
@@ -63,9 +65,7 @@ def test_update_stress(named_port, logger):
         updaters = []
         for i in range(5):
             zone = f"zone00000{i}.example."
-            updaters.append(
-                executor.submit(update_zone, test_state, zone, named_port, logger)
-            )
+            updaters.append(executor.submit(update_zone, test_state, zone, named_port))
 
         # All the update_zone() tasks are expected to complete within 5
         # minutes.  If they do not, we cannot assert immediately as that will
