@@ -14,6 +14,7 @@
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 #include <isc/attributes.h>
 #include <isc/base32.h>
@@ -35,7 +36,7 @@
 
 const char *program = "nsec3hash";
 
-ISC_NORETURN static void
+noreturn static void
 fatal(const char *format, ...);
 
 static void
@@ -47,7 +48,7 @@ fatal(const char *format, ...) {
 	vfprintf(stderr, format, args);
 	va_end(args);
 	fprintf(stderr, "\n");
-	exit(1);
+	_exit(EXIT_FAILURE);
 }
 
 static void
@@ -63,7 +64,7 @@ usage(void) {
 		program);
 	fprintf(stderr, "       %s -r algorithm flags iterations salt domain\n",
 		program);
-	exit(1);
+	exit(EXIT_FAILURE);
 }
 
 typedef void
