@@ -39,7 +39,7 @@ check_result(isc_result_t result, const char *format, ...) {
 	vfprintf(stderr, format, args);
 	va_end(args);
 	fprintf(stderr, ": %s\n", isc_result_totext(result));
-	exit(1);
+	exit(EXIT_FAILURE);
 }
 
 static void
@@ -53,7 +53,7 @@ usage(void) {
 	fprintf(stderr, "usage: cfg_test --rndc|--named "
 			"[--grammar] [--zonegrammar] [--active] "
 			"[--memstats] conffile\n");
-	exit(1);
+	exit(EXIT_FAILURE);
 }
 
 int
@@ -110,10 +110,12 @@ main(int argc, char **argv) {
 				usage();
 			}
 			if (strcmp(argv[1], "master") == 0 ||
-			    strcmp(argv[1], "primary") == 0) {
+			    strcmp(argv[1], "primary") == 0)
+			{
 				zonetype = CFG_ZONE_PRIMARY;
 			} else if (strcmp(argv[1], "slave") == 0 ||
-				   strcmp(argv[1], "secondary") == 0) {
+				   strcmp(argv[1], "secondary") == 0)
+			{
 				zonetype = CFG_ZONE_SECONDARY;
 			} else if (strcmp(argv[1], "mirror") == 0) {
 				zonetype = CFG_ZONE_MIRROR;
@@ -167,7 +169,7 @@ main(int argc, char **argv) {
 		fprintf(stderr, "read config: %s\n", isc_result_totext(result));
 
 		if (result != ISC_R_SUCCESS) {
-			exit(1);
+			exit(EXIT_FAILURE);
 		}
 
 		cfg_print(cfg, output, NULL);
@@ -187,7 +189,7 @@ main(int argc, char **argv) {
 	if (ferror(stdout)) {
 		fprintf(stderr, "write error\n");
 		return (1);
-	} else {
-		return (0);
 	}
+
+	return (0);
 }
