@@ -1,9 +1,11 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
+ * SPDX-License-Identifier: MPL-2.0
+ *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * file, you can obtain one at https://mozilla.org/MPL/2.0/.
  *
  * See the COPYRIGHT file distributed with this work for additional
  * information regarding copyright ownership.
@@ -130,12 +132,13 @@ again:
 			goto again;
 		}
 #endif /* ifdef AI_ADDRCONFIG */
-	/* FALLTHROUGH */
+		FALLTHROUGH;
 	default:
 		return (ISC_R_FAILURE);
 	}
 	for (tmpai = ai, i = 0; tmpai != NULL && i < addrsize;
-	     tmpai = tmpai->ai_next) {
+	     tmpai = tmpai->ai_next)
+	{
 		if (tmpai->ai_family != AF_INET && tmpai->ai_family != AF_INET6)
 		{
 			continue;
