@@ -32,6 +32,7 @@
 
 #include <isc/buffer.h>
 #include <isc/hmac.h>
+#include <isc/lex.h>
 #include <isc/md.h>
 #include <isc/mem.h>
 #include <isc/nonce.h>
@@ -102,7 +103,17 @@
 	}                                                                      \
 	static isc_result_t hmac##alg##_parse(                                 \
 		dst_key_t *key, isc_lex_t *lexer, dst_key_t *pub) {            \
-		return (hmac_parse(ISC_MD_##alg, key, lexer, pub));            \
+		const char *file = isc_lex_getsourcename(lexer);               \
+		isc_result_t result;                                           \
+		result = hmac_parse(ISC_MD_##alg, key, lexer, pub);            \
+		if (result == ISC_R_SUCCESS && file != NULL) {                 \
+			isc_log_write(dns_lctx, DNS_LOGCATEGORY_GENERAL,       \
+				      DNS_LOGMODULE_CRYPTO, ISC_LOG_WARNING,   \
+				      "%s: Use of K* file pairs for HMAC is "  \
+				      "deprecated\n",                          \
+				      file);                                   \
+		}                                                              \
+		return (result);                                               \
 	}                                                                      \
 	static dst_func_t hmac##alg##_functions = {                            \
 		hmac##alg##_createctx,                                         \
@@ -142,7 +153,7 @@ struct dst_hmac_key {
 	uint8_t key[ISC_MAX_BLOCK_SIZE];
 };
 
-static inline isc_result_t
+static isc_result_t
 getkeybits(dst_key_t *key, struct dst_private_element *element) {
 	uint16_t *bits = (uint16_t *)element->data;
 
@@ -155,7 +166,7 @@ getkeybits(dst_key_t *key, struct dst_private_element *element) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 hmac_createctx(const isc_md_type_t *type, const dst_key_t *key,
 	       dst_context_t *dctx) {
 	isc_result_t result;
@@ -165,6 +176,7 @@ hmac_createctx(const isc_md_type_t *type, const dst_key_t *key,
 	result = isc_hmac_init(ctx, hkey->key, isc_md_type_get_block_size(type),
 			       type);
 	if (result != ISC_R_SUCCESS) {
+		isc_hmac_free(ctx);
 		return (DST_R_UNSUPPORTEDALG);
 	}
 
@@ -172,7 +184,7 @@ hmac_createctx(const isc_md_type_t *type, const dst_key_t *key,
 	return (ISC_R_SUCCESS);
 }
 
-static inline void
+static void
 hmac_destroyctx(dst_context_t *dctx) {
 	isc_hmac_t *ctx = dctx->ctxdata.hmac_ctx;
 	REQUIRE(ctx != NULL);
@@ -181,7 +193,7 @@ hmac_destroyctx(dst_context_t *dctx) {
 	dctx->ctxdata.hmac_ctx = NULL;
 }
 
-static inline isc_result_t
+static isc_result_t
 hmac_adddata(const dst_context_t *dctx, const isc_region_t *data) {
 	isc_result_t result;
 	isc_hmac_t *ctx = dctx->ctxdata.hmac_ctx;
@@ -196,7 +208,7 @@ hmac_adddata(const dst_context_t *dctx, const isc_region_t *data) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 hmac_sign(const dst_context_t *dctx, isc_buffer_t *sig) {
 	isc_hmac_t *ctx = dctx->ctxdata.hmac_ctx;
 	REQUIRE(ctx != NULL);
@@ -220,7 +232,7 @@ hmac_sign(const dst_context_t *dctx, isc_buffer_t *sig) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 hmac_verify(const dst_context_t *dctx, const isc_region_t *sig) {
 	isc_hmac_t *ctx = dctx->ctxdata.hmac_ctx;
 	unsigned char digest[ISC_MAX_MD_SIZE];
@@ -245,7 +257,7 @@ hmac_verify(const dst_context_t *dctx, const isc_region_t *sig) {
 			: DST_R_VERIFYFAILURE);
 }
 
-static inline bool
+static bool
 hmac_compare(const isc_md_type_t *type, const dst_key_t *key1,
 	     const dst_key_t *key2) {
 	dst_hmac_key_t *hkey1, *hkey2;
@@ -263,7 +275,7 @@ hmac_compare(const isc_md_type_t *type, const dst_key_t *key1,
 				  isc_md_type_get_block_size(type)));
 }
 
-static inline isc_result_t
+static isc_result_t
 hmac_generate(const isc_md_type_t *type, dst_key_t *key) {
 	isc_buffer_t b;
 	isc_result_t ret;
@@ -291,13 +303,13 @@ hmac_generate(const isc_md_type_t *type, dst_key_t *key) {
 	return (ret);
 }
 
-static inline bool
+static bool
 hmac_isprivate(const dst_key_t *key) {
 	UNUSED(key);
 	return (true);
 }
 
-static inline void
+static void
 hmac_destroy(dst_key_t *key) {
 	dst_hmac_key_t *hkey = key->keydata.hmac_key;
 	isc_safe_memwipe(hkey, sizeof(*hkey));
@@ -305,7 +317,7 @@ hmac_destroy(dst_key_t *key) {
 	key->keydata.hmac_key = NULL;
 }
 
-static inline isc_result_t
+static isc_result_t
 hmac_todns(const dst_key_t *key, isc_buffer_t *data) {
 	REQUIRE(key != NULL && key->keydata.hmac_key != NULL);
 	dst_hmac_key_t *hkey = key->keydata.hmac_key;
@@ -320,7 +332,7 @@ hmac_todns(const dst_key_t *key, isc_buffer_t *data) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 hmac_fromdns(const isc_md_type_t *type, dst_key_t *key, isc_buffer_t *data) {
 	dst_hmac_key_t *hkey;
 	unsigned int keylen;
@@ -338,7 +350,8 @@ hmac_fromdns(const isc_md_type_t *type, dst_key_t *key, isc_buffer_t *data) {
 	/* Hash the key if the key is longer then chosen MD block size */
 	if (r.length > (unsigned int)isc_md_type_get_block_size(type)) {
 		if (isc_md(type, r.base, r.length, hkey->key, &keylen) !=
-		    ISC_R_SUCCESS) {
+		    ISC_R_SUCCESS)
+		{
 			isc_mem_put(key->mctx, hkey, sizeof(dst_hmac_key_t));
 			return (DST_R_OPENSSLFAILURE);
 		}
@@ -355,7 +368,7 @@ hmac_fromdns(const isc_md_type_t *type, dst_key_t *key, isc_buffer_t *data) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline int
+static int
 hmac__get_tag_key(const isc_md_type_t *type) {
 	if (type == ISC_MD_MD5) {
 		return (TAG_HMACMD5_KEY);
@@ -370,12 +383,11 @@ hmac__get_tag_key(const isc_md_type_t *type) {
 	} else if (type == ISC_MD_SHA512) {
 		return (TAG_HMACSHA512_KEY);
 	} else {
-		INSIST(0);
-		ISC_UNREACHABLE();
+		UNREACHABLE();
 	}
 }
 
-static inline int
+static int
 hmac__get_tag_bits(const isc_md_type_t *type) {
 	if (type == ISC_MD_MD5) {
 		return (TAG_HMACMD5_BITS);
@@ -390,12 +402,11 @@ hmac__get_tag_bits(const isc_md_type_t *type) {
 	} else if (type == ISC_MD_SHA512) {
 		return (TAG_HMACSHA512_BITS);
 	} else {
-		INSIST(0);
-		ISC_UNREACHABLE();
+		UNREACHABLE();
 	}
 }
 
-static inline isc_result_t
+static isc_result_t
 hmac_tofile(const isc_md_type_t *type, const dst_key_t *key,
 	    const char *directory) {
 	dst_hmac_key_t *hkey;
@@ -428,7 +439,7 @@ hmac_tofile(const isc_md_type_t *type, const dst_key_t *key,
 	return (dst__privstruct_writefile(key, &priv, directory));
 }
 
-static inline int
+static int
 hmac__to_dst_alg(const isc_md_type_t *type) {
 	if (type == ISC_MD_MD5) {
 		return (DST_ALG_HMACMD5);
@@ -443,12 +454,11 @@ hmac__to_dst_alg(const isc_md_type_t *type) {
 	} else if (type == ISC_MD_SHA512) {
 		return (DST_ALG_HMACSHA512);
 	} else {
-		INSIST(0);
-		ISC_UNREACHABLE();
+		UNREACHABLE();
 	}
 }
 
-static inline isc_result_t
+static isc_result_t
 hmac_parse(const isc_md_type_t *type, dst_key_t *key, isc_lex_t *lexer,
 	   dst_key_t *pub) {
 	dst_private_t priv;
