@@ -23,6 +23,7 @@
 #include <isc/mem.h>
 #include <isc/print.h>
 #include <isc/refcount.h>
+#include <isc/result.h>
 #include <isc/serial.h>
 #include <isc/stdio.h>
 #include <isc/stdtime.h>
@@ -41,7 +42,6 @@
 #include <dns/rdataset.h>
 #include <dns/rdatastruct.h>
 #include <dns/rdatatype.h>
-#include <dns/result.h>
 #include <dns/soa.h>
 #include <dns/time.h>
 #include <dns/ttl.h>
@@ -187,12 +187,6 @@ openfile_raw(dns_loadctx_t *lctx, const char *master_file);
 static isc_result_t
 load_raw(dns_loadctx_t *lctx);
 
-static isc_result_t
-openfile_map(dns_loadctx_t *lctx, const char *master_file);
-
-static isc_result_t
-load_map(dns_loadctx_t *lctx);
-
 static isc_result_t
 pushfile(const char *master_file, dns_name_t *origin, dns_loadctx_t *lctx);
 
@@ -220,33 +214,52 @@ task_send(dns_loadctx_t *lctx);
 static void
 loadctx_destroy(dns_loadctx_t *lctx);
 
-#define GETTOKENERR(lexer, options, token, eol, err)                      \
-	do {                                                              \
-		result = gettoken(lexer, options, token, eol, callbacks); \
-		switch (result) {                                         \
-		case ISC_R_SUCCESS:                                       \
-			break;                                            \
-		case ISC_R_UNEXPECTED:                                    \
-			goto insist_and_cleanup;                          \
-		default:                                                  \
-			if (MANYERRS(lctx, result)) {                     \
-				SETRESULT(lctx, result);                  \
-				LOGIT(result);                            \
-				read_till_eol = true;                     \
-				err goto next_line;                       \
-			} else                                            \
-				goto log_and_cleanup;                     \
-		}                                                         \
-		if ((token)->type == isc_tokentype_special) {             \
-			result = DNS_R_SYNTAX;                            \
-			if (MANYERRS(lctx, result)) {                     \
-				SETRESULT(lctx, result);                  \
-				LOGIT(result);                            \
-				read_till_eol = true;                     \
-				goto next_line;                           \
-			} else                                            \
-				goto log_and_cleanup;                     \
-		}                                                         \
+#define LCTX_MANYERRORS(lctx) (((lctx)->options & DNS_MASTER_MANYERRORS) != 0)
+
+#define GETTOKENERR(lexer, options, token, eol, err)                         \
+	do {                                                                 \
+		result = gettoken(lexer, options, token, eol, callbacks);    \
+		switch (result) {                                            \
+		case ISC_R_SUCCESS:                                          \
+			break;                                               \
+		case ISC_R_NOTFILE:                                          \
+			/* Treat "bad" $INCLUDE as eof. */                   \
+			if (ictx->parent != NULL && LCTX_MANYERRORS(lctx)) { \
+				SETRESULT(lctx, result);                     \
+				COMMITALL;                                   \
+				lctx->inc = ictx->parent;                    \
+				ictx->parent = NULL;                         \
+				incctx_destroy(lctx->mctx, ictx);            \
+				RUNTIME_CHECK(isc_lex_close(lctx->lex) ==    \
+					      ISC_R_SUCCESS);                \
+				line = isc_lex_getsourceline(lctx->lex);     \
+				POST(line);                                  \
+				source = isc_lex_getsourcename(lctx->lex);   \
+				ictx = lctx->inc;                            \
+				continue;                                    \
+			}                                                    \
+			goto insist_and_cleanup;                             \
+		case ISC_R_UNEXPECTED:                                       \
+			goto insist_and_cleanup;                             \
+		default:                                                     \
+			if (MANYERRS(lctx, result)) {                        \
+				SETRESULT(lctx, result);                     \
+				LOGIT(result);                               \
+				read_till_eol = true;                        \
+				err goto next_line;                          \
+			} else                                               \
+				goto log_and_cleanup;                        \
+		}                                                            \
+		if ((token)->type == isc_tokentype_special) {                \
+			result = DNS_R_SYNTAX;                               \
+			if (MANYERRS(lctx, result)) {                        \
+				SETRESULT(lctx, result);                     \
+				LOGIT(result);                               \
+				read_till_eol = true;                        \
+				goto next_line;                              \
+			} else                                               \
+				goto log_and_cleanup;                        \
+		}                                                            \
 	} while (0)
 #define GETTOKEN(lexer, options, token, eol) \
 	GETTOKENERR(lexer, options, token, eol, {})
@@ -299,7 +312,7 @@ loadctx_destroy(dns_loadctx_t *lctx);
 
 #define MANYERRS(lctx, result)                                     \
 	((result != ISC_R_SUCCESS) && (result != ISC_R_IOERROR) && \
-	 ((lctx)->options & DNS_MASTER_MANYERRORS) != 0)
+	 LCTX_MANYERRORS(lctx))
 
 #define SETRESULT(lctx, r)                           \
 	do {                                         \
@@ -313,18 +326,18 @@ loadctx_destroy(dns_loadctx_t *lctx);
 	    result == ISC_R_NOPERM)                                            \
 		(*callbacks->error)(callbacks, "%s: %s:%lu: %s: %s",           \
 				    "dns_master_load", source, line, filename, \
-				    dns_result_totext(result));                \
+				    isc_result_totext(result));                \
 	else                                                                   \
 		LOGIT(result)
 
 #define LOGIT(result)                                                 \
 	if (result == ISC_R_NOMEMORY)                                 \
 		(*callbacks->error)(callbacks, "dns_master_load: %s", \
-				    dns_result_totext(result));       \
+				    isc_result_totext(result));       \
 	else                                                          \
 		(*callbacks->error)(callbacks, "%s: %s:%lu: %s",      \
 				    "dns_master_load", source, line,  \
-				    dns_result_totext(result))
+				    isc_result_totext(result))
 
 static unsigned char in_addr_arpa_data[] = "\007IN-ADDR\004ARPA";
 static unsigned char in_addr_arpa_offsets[] = { 0, 8, 13 };
@@ -344,7 +357,7 @@ static dns_name_t const ip6_arpa = DNS_NAME_INITABSOLUTE(ip6_arpa_data,
 static bool
 dns_master_isprimary(dns_loadctx_t *lctx) {
 	return ((lctx->options & DNS_MASTER_ZONE) != 0 &&
-		(lctx->options & DNS_MASTER_SLAVE) == 0 &&
+		(lctx->options & DNS_MASTER_SECONDARY) == 0 &&
 		(lctx->options & DNS_MASTER_KEY) == 0);
 }
 
@@ -452,8 +465,7 @@ loadctx_destroy(dns_loadctx_t *lctx) {
 	if (lctx->f != NULL) {
 		isc_result_t result = isc_stdio_close(lctx->f);
 		if (result != ISC_R_SUCCESS) {
-			UNEXPECTED_ERROR(__FILE__, __LINE__,
-					 "isc_stdio_close() failed: %s",
+			UNEXPECTED_ERROR("isc_stdio_close() failed: %s",
 					 isc_result_totext(result));
 		}
 	}
@@ -546,10 +558,6 @@ loadctx_create(dns_masterformat_t format, isc_mem_t *mctx, unsigned int options,
 		lctx->openfile = openfile_raw;
 		lctx->load = load_raw;
 		break;
-	case dns_masterformat_map:
-		lctx->openfile = openfile_map;
-		lctx->load = load_map;
-		break;
 	default:
 		UNREACHABLE();
 	}
@@ -637,7 +645,8 @@ static const char *hex = "0123456789abcdef0123456789ABCDEF";
  * counting the terminating NUL.
  */
 static unsigned int
-nibbles(char *numbuf, size_t length, unsigned int width, char mode, int value) {
+nibbles(char *numbuf, size_t length, unsigned int width, char mode,
+	unsigned int value) {
 	unsigned int count = 0;
 
 	/*
@@ -946,10 +955,10 @@ generate(dns_loadctx_t *lctx, char *range, char *lhs, char *gtype, char *rhs,
 error_cleanup:
 	if (result == ISC_R_NOMEMORY) {
 		(*callbacks->error)(callbacks, "$GENERATE: %s",
-				    dns_result_totext(result));
+				    isc_result_totext(result));
 	} else {
 		(*callbacks->error)(callbacks, "$GENERATE: %s:%lu: %s", source,
-				    line, dns_result_totext(result));
+				    line, isc_result_totext(result));
 	}
 
 insist_cleanup:
@@ -1295,8 +1304,7 @@ load_text(dns_loadctx_t *lctx) {
 				}
 				dump_time = (isc_stdtime_t)dump_time64;
 				if (dump_time != dump_time64) {
-					UNEXPECTED_ERROR(__FILE__, __LINE__,
-							 "%s: %s:%lu: $DATE "
+					UNEXPECTED_ERROR("%s: %s:%lu: $DATE "
 							 "outside epoch",
 							 "dns_master_load",
 							 source, line);
@@ -1304,8 +1312,7 @@ load_text(dns_loadctx_t *lctx) {
 					goto insist_and_cleanup;
 				}
 				if (dump_time > current_time) {
-					UNEXPECTED_ERROR(__FILE__, __LINE__,
-							 "%s: %s:%lu: "
+					UNEXPECTED_ERROR("%s: %s:%lu: "
 							 "$DATE in future, "
 							 "using current date",
 							 "dns_master_load",
@@ -1592,8 +1599,7 @@ load_text(dns_loadctx_t *lctx) {
 				ictx->drop = false;
 			}
 		} else {
-			UNEXPECTED_ERROR(__FILE__, __LINE__,
-					 "%s:%lu: isc_lex_gettoken() returned "
+			UNEXPECTED_ERROR("%s:%lu: isc_lex_gettoken() returned "
 					 "unexpected token type (%d)",
 					 source, line, token.type);
 			result = ISC_R_UNEXPECTED;
@@ -1685,8 +1691,7 @@ load_text(dns_loadctx_t *lctx) {
 		}
 
 		if (token.type != isc_tokentype_string) {
-			UNEXPECTED_ERROR(__FILE__, __LINE__,
-					 "isc_lex_gettoken() returned "
+			UNEXPECTED_ERROR("isc_lex_gettoken() returned "
 					 "unexpected token type");
 			result = ISC_R_UNEXPECTED;
 			if (MANYERRS(lctx, result)) {
@@ -1707,8 +1712,7 @@ load_text(dns_loadctx_t *lctx) {
 		}
 
 		if (token.type != isc_tokentype_string) {
-			UNEXPECTED_ERROR(__FILE__, __LINE__,
-					 "isc_lex_gettoken() returned "
+			UNEXPECTED_ERROR("isc_lex_gettoken() returned "
 					 "unexpected token type");
 			result = ISC_R_UNEXPECTED;
 			if (MANYERRS(lctx, result)) {
@@ -1780,7 +1784,7 @@ load_text(dns_loadctx_t *lctx) {
 			dns_rdatatype_format(type, typebuf, sizeof(typebuf));
 			(*callbacks->error)(callbacks, "%s:%lu: %s '%s': %s",
 					    source, line, "type", typebuf,
-					    dns_result_totext(result));
+					    isc_result_totext(result));
 			if (MANYERRS(lctx, result)) {
 				SETRESULT(lctx, result);
 			} else {
@@ -1800,7 +1804,7 @@ load_text(dns_loadctx_t *lctx) {
 			dns_rdatatype_format(type, typebuf, sizeof(typebuf));
 			(*callbacks->error)(callbacks, "%s:%lu: %s '%s': %s",
 					    source, line, "type", typebuf,
-					    dns_result_totext(result));
+					    isc_result_totext(result));
 			if (MANYERRS(lctx, result)) {
 				SETRESULT(lctx, result);
 			} else {
@@ -1859,7 +1863,7 @@ load_text(dns_loadctx_t *lctx) {
 				const char *desc;
 				dns_name_format(name, namebuf, sizeof(namebuf));
 				result = DNS_R_BADOWNERNAME;
-				desc = dns_result_totext(result);
+				desc = isc_result_totext(result);
 				if (CHECKNAMESFAIL(lctx->options) ||
 				    type == dns_rdatatype_nsec3)
 				{
@@ -2285,9 +2289,7 @@ load_header(dns_loadctx_t *lctx) {
 
 	REQUIRE(DNS_LCTX_VALID(lctx));
 
-	if (lctx->format != dns_masterformat_raw &&
-	    lctx->format != dns_masterformat_map)
-	{
+	if (lctx->format != dns_masterformat_raw) {
 		return (ISC_R_NOTIMPLEMENTED);
 	}
 
@@ -2299,8 +2301,7 @@ load_header(dns_loadctx_t *lctx) {
 
 	result = isc_stdio_read(data, 1, commonlen, lctx->f, NULL);
 	if (result != ISC_R_SUCCESS) {
-		UNEXPECTED_ERROR(__FILE__, __LINE__,
-				 "isc_stdio_read failed: %s",
+		UNEXPECTED_ERROR("isc_stdio_read failed: %s",
 				 isc_result_totext(result));
 		return (result);
 	}
@@ -2310,10 +2311,7 @@ load_header(dns_loadctx_t *lctx) {
 	if (header.format != lctx->format) {
 		(*callbacks->error)(callbacks,
 				    "dns_master_load: "
-				    "file format mismatch (not %s)",
-				    lctx->format == dns_masterformat_map ? "map"
-									 : "ra"
-									   "w");
+				    "file format mismatch (not raw)");
 		return (ISC_R_NOTIMPLEMENTED);
 	}
 
@@ -2335,8 +2333,7 @@ load_header(dns_loadctx_t *lctx) {
 
 	result = isc_stdio_read(data + commonlen, 1, remainder, lctx->f, NULL);
 	if (result != ISC_R_SUCCESS) {
-		UNEXPECTED_ERROR(__FILE__, __LINE__,
-				 "isc_stdio_read failed: %s",
+		UNEXPECTED_ERROR("isc_stdio_read failed: %s",
 				 isc_result_totext(result));
 		return (result);
 	}
@@ -2355,54 +2352,13 @@ load_header(dns_loadctx_t *lctx) {
 	return (ISC_R_SUCCESS);
 }
 
-static isc_result_t
-openfile_map(dns_loadctx_t *lctx, const char *master_file) {
-	isc_result_t result;
-
-	result = isc_stdio_open(master_file, "rb", &lctx->f);
-	if (result != ISC_R_SUCCESS && result != ISC_R_FILENOTFOUND) {
-		UNEXPECTED_ERROR(__FILE__, __LINE__,
-				 "isc_stdio_open() failed: %s",
-				 isc_result_totext(result));
-	}
-
-	return (result);
-}
-
-/*
- * Load a map format file, using mmap() to access RBT trees directly
- */
-static isc_result_t
-load_map(dns_loadctx_t *lctx) {
-	isc_result_t result = ISC_R_SUCCESS;
-	dns_rdatacallbacks_t *callbacks;
-
-	REQUIRE(DNS_LCTX_VALID(lctx));
-
-	callbacks = lctx->callbacks;
-
-	if (lctx->first) {
-		result = load_header(lctx);
-		if (result != ISC_R_SUCCESS) {
-			return (result);
-		}
-
-		result = (*callbacks->deserialize)(
-			callbacks->deserialize_private, lctx->f,
-			sizeof(dns_masterrawheader_t));
-	}
-
-	return (result);
-}
-
 static isc_result_t
 openfile_raw(dns_loadctx_t *lctx, const char *master_file) {
 	isc_result_t result;
 
 	result = isc_stdio_open(master_file, "rb", &lctx->f);
 	if (result != ISC_R_SUCCESS && result != ISC_R_FILENOTFOUND) {
-		UNEXPECTED_ERROR(__FILE__, __LINE__,
-				 "isc_stdio_open() failed: %s",
+		UNEXPECTED_ERROR("isc_stdio_open() failed: %s",
 				 isc_result_totext(result));
 	}
 
@@ -2705,7 +2661,7 @@ cleanup:
 	}
 	if (result != ISC_R_SUCCESS && result != DNS_R_CONTINUE) {
 		(*callbacks->error)(callbacks, "dns_master_load: %s",
-				    dns_result_totext(result));
+				    isc_result_totext(result));
 	}
 
 	return (result);
@@ -3155,17 +3111,17 @@ commit(dns_rdatacallbacks_t *callbacks, dns_loadctx_t *lctx,
 					    &dataset));
 		if (result == ISC_R_NOMEMORY) {
 			(*error)(callbacks, "dns_master_load: %s",
-				 dns_result_totext(result));
+				 isc_result_totext(result));
 		} else if (result != ISC_R_SUCCESS) {
 			dns_name_format(owner, namebuf, sizeof(namebuf));
 			if (source != NULL) {
 				(*error)(callbacks, "%s: %s:%lu: %s: %s",
 					 "dns_master_load", source, line,
-					 namebuf, dns_result_totext(result));
+					 namebuf, isc_result_totext(result));
 			} else {
 				(*error)(callbacks, "%s: %s: %s",
 					 "dns_master_load", namebuf,
-					 dns_result_totext(result));
+					 isc_result_totext(result));
 			}
 		}
 		if (MANYERRS(lctx, result)) {
