@@ -63,7 +63,7 @@ enable_fips_mode(void) {
 
 	if (FIPS_mode_set(1) == 0) {
 		dst__openssl_toresult2("FIPS_mode_set", DST_R_OPENSSLFAILURE);
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 #endif /* HAVE_FIPS_MODE */
 }
@@ -104,6 +104,7 @@ cleanup_rm:
 		ENGINE_free(e);
 	}
 	e = NULL;
+	ERR_clear_error();
 #else
 	UNUSED(engine);
 #endif /* if !defined(OPENSSL_NO_ENGINE) && OPENSSL_API_LEVEL < 30000 */
