@@ -23,6 +23,7 @@
 #include <isc/mem.h>
 #include <isc/parseint.h>
 #include <isc/print.h>
+#include <isc/result.h>
 #include <isc/string.h>
 #include <isc/util.h>
 
@@ -31,15 +32,16 @@
 #include <dns/compress.h>
 #include <dns/dsdigest.h>
 #include <dns/enumtype.h>
+#include <dns/fixedname.h>
 #include <dns/keyflags.h>
 #include <dns/keyvalues.h>
 #include <dns/message.h>
 #include <dns/rcode.h>
 #include <dns/rdata.h>
 #include <dns/rdataclass.h>
+#include <dns/rdataset.h>
 #include <dns/rdatastruct.h>
 #include <dns/rdatatype.h>
-#include <dns/result.h>
 #include <dns/secalg.h>
 #include <dns/secproto.h>
 #include <dns/time.h>
@@ -120,10 +122,11 @@
 
 #define CALL_FREESTRUCT source
 
-#define ARGS_ADDLDATA \
-	dns_rdata_t *rdata, dns_additionaldatafunc_t add, void *arg
+#define ARGS_ADDLDATA                                \
+	dns_rdata_t *rdata, const dns_name_t *owner, \
+		dns_additionaldatafunc_t add, void *arg
 
-#define CALL_ADDLDATA rdata, add, arg
+#define CALL_ADDLDATA rdata, owner, add, arg
 
 #define ARGS_DIGEST dns_rdata_t *rdata, dns_digestfunc_t digest, void *arg
 
@@ -406,14 +409,13 @@ locator_pton(const char *src, unsigned char *dst) {
 	return (1);
 }
 
-static isc_result_t
+static void
 name_duporclone(const dns_name_t *source, isc_mem_t *mctx, dns_name_t *target) {
 	if (mctx != NULL) {
 		dns_name_dup(source, mctx, target);
 	} else {
 		dns_name_clone(source, target);
 	}
-	return (ISC_R_SUCCESS);
 }
 
 static void *
@@ -1271,8 +1273,8 @@ dns_rdata_freestruct(void *source) {
 }
 
 isc_result_t
-dns_rdata_additionaldata(dns_rdata_t *rdata, dns_additionaldatafunc_t add,
-			 void *arg) {
+dns_rdata_additionaldata(dns_rdata_t *rdata, const dns_name_t *owner,
+			 dns_additionaldatafunc_t add, void *arg) {
 	isc_result_t result = ISC_R_NOTIMPLEMENTED;
 	bool use_default = false;
 
@@ -1637,21 +1639,26 @@ commatxt_fromtext(isc_textregion_t *source, bool comma, isc_buffer_t *target) {
 
 	if (comma) {
 		/*
-		 * Disallow empty ALPN at start (",h1") or in the
-		 * middle ("h1,,h2").
+		 * Disallow empty ALPN at start (",h1" or "\,h1") or
+		 * in the middle ("h1,,h2" or "h1\,\,h2").
 		 */
-		if (s == source->base || (seen_comma && s == source->base + 1))
-		{
+		if ((t - tregion.base - 1) == 0) {
 			return (DNS_R_SYNTAX);
 		}
-		isc_textregion_consume(source, s - source->base);
+
 		/*
-		 * Disallow empty ALPN at end ("h1,").
+		 * Consume this ALPN and possible ending comma.
+		 */
+		isc_textregion_consume(source, s - source->base);
+
+		/*
+		 * Disallow empty ALPN at end ("h1," or "h1\,").
 		 */
 		if (seen_comma && source->length == 0) {
 			return (DNS_R_SYNTAX);
 		}
 	}
+
 	*tregion.base = (unsigned char)(t - tregion.base - 1);
 	isc_buffer_add(target, *tregion.base + 1);
 	return (ISC_R_SUCCESS);
@@ -2103,7 +2110,7 @@ warn_badmx(isc_token_t *token, isc_lex_t *lexer,
 		line = isc_lex_getsourceline(lexer);
 		(*callbacks->warn)(callbacks, "%s:%u: warning: '%s': %s", file,
 				   line, DNS_AS_STR(*token),
-				   dns_result_totext(DNS_R_MXISADDRESS));
+				   isc_result_totext(DNS_R_MXISADDRESS));
 	}
 }
 
@@ -2120,7 +2127,7 @@ warn_badname(const dns_name_t *name, isc_lex_t *lexer,
 		dns_name_format(name, namebuf, sizeof(namebuf));
 		(*callbacks->warn)(callbacks, "%s:%u: warning: %s: %s", file,
 				   line, namebuf,
-				   dns_result_totext(DNS_R_BADNAME));
+				   isc_result_totext(DNS_R_BADNAME));
 	}
 }
 
@@ -2137,35 +2144,35 @@ fromtext_error(void (*callback)(dns_rdatacallbacks_t *, const char *, ...),
 		case isc_tokentype_eol:
 			(*callback)(callbacks, "%s: %s:%lu: near eol: %s",
 				    "dns_rdata_fromtext", name, line,
-				    dns_result_totext(result));
+				    isc_result_totext(result));
 			break;
 		case isc_tokentype_eof:
 			(*callback)(callbacks, "%s: %s:%lu: near eof: %s",
 				    "dns_rdata_fromtext", name, line,
-				    dns_result_totext(result));
+				    isc_result_totext(result));
 			break;
 		case isc_tokentype_number:
 			(*callback)(callbacks, "%s: %s:%lu: near %lu: %s",
 				    "dns_rdata_fromtext", name, line,
 				    token->value.as_ulong,
-				    dns_result_totext(result));
+				    isc_result_totext(result));
 			break;
 		case isc_tokentype_string:
 		case isc_tokentype_qstring:
 			(*callback)(callbacks, "%s: %s:%lu: near '%s': %s",
 				    "dns_rdata_fromtext", name, line,
 				    DNS_AS_STR(*token),
-				    dns_result_totext(result));
+				    isc_result_totext(result));
 			break;
 		default:
 			(*callback)(callbacks, "%s: %s:%lu: %s",
 				    "dns_rdata_fromtext", name, line,
-				    dns_result_totext(result));
+				    isc_result_totext(result));
 			break;
 		}
 	} else {
 		(*callback)(callbacks, "dns_rdata_fromtext: %s:%lu: %s", name,
-			    line, dns_result_totext(result));
+			    line, isc_result_totext(result));
 	}
 }
 
@@ -2260,6 +2267,12 @@ dns_rdatatype_isdnssec(dns_rdatatype_t type) {
 	return (false);
 }
 
+bool
+dns_rdatatype_iskeymaterial(dns_rdatatype_t type) {
+	return (type == dns_rdatatype_dnskey || type == dns_rdatatype_cdnskey ||
+		type == dns_rdatatype_cds);
+}
+
 bool
 dns_rdatatype_iszonecutauth(dns_rdatatype_t type) {
 	if ((dns_rdatatype_attributes(type) & DNS_RDATATYPEATTR_ZONECUTAUTH) !=
