@@ -18,7 +18,7 @@
 
 #define RRTYPE_EUI48_ATTRIBUTES (0)
 
-static inline isc_result_t
+static isc_result_t
 fromtext_eui48(ARGS_FROMTEXT) {
 	isc_token_t token;
 	unsigned char eui48[6];
@@ -52,7 +52,7 @@ fromtext_eui48(ARGS_FROMTEXT) {
 	return (mem_tobuffer(target, eui48, sizeof(eui48)));
 }
 
-static inline isc_result_t
+static isc_result_t
 totext_eui48(ARGS_TOTEXT) {
 	char buf[sizeof("xx-xx-xx-xx-xx-xx")];
 
@@ -67,7 +67,7 @@ totext_eui48(ARGS_TOTEXT) {
 	return (str_totext(buf, target));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromwire_eui48(ARGS_FROMWIRE) {
 	isc_region_t sregion;
 
@@ -86,7 +86,7 @@ fromwire_eui48(ARGS_FROMWIRE) {
 	return (mem_tobuffer(target, sregion.base, sregion.length));
 }
 
-static inline isc_result_t
+static isc_result_t
 towire_eui48(ARGS_TOWIRE) {
 	REQUIRE(rdata->type == dns_rdatatype_eui48);
 	REQUIRE(rdata->length == 6);
@@ -96,7 +96,7 @@ towire_eui48(ARGS_TOWIRE) {
 	return (mem_tobuffer(target, rdata->data, rdata->length));
 }
 
-static inline int
+static int
 compare_eui48(ARGS_COMPARE) {
 	isc_region_t region1;
 	isc_region_t region2;
@@ -112,7 +112,7 @@ compare_eui48(ARGS_COMPARE) {
 	return (isc_region_compare(&region1, &region2));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromstruct_eui48(ARGS_FROMSTRUCT) {
 	dns_rdata_eui48_t *eui48 = source;
 
@@ -127,7 +127,7 @@ fromstruct_eui48(ARGS_FROMSTRUCT) {
 	return (mem_tobuffer(target, eui48->eui48, sizeof(eui48->eui48)));
 }
 
-static inline isc_result_t
+static isc_result_t
 tostruct_eui48(ARGS_TOSTRUCT) {
 	dns_rdata_eui48_t *eui48 = target;
 
@@ -145,7 +145,7 @@ tostruct_eui48(ARGS_TOSTRUCT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline void
+static void
 freestruct_eui48(ARGS_FREESTRUCT) {
 	dns_rdata_eui48_t *eui48 = source;
 
@@ -155,7 +155,7 @@ freestruct_eui48(ARGS_FREESTRUCT) {
 	return;
 }
 
-static inline isc_result_t
+static isc_result_t
 additionaldata_eui48(ARGS_ADDLDATA) {
 	REQUIRE(rdata->type == dns_rdatatype_eui48);
 	REQUIRE(rdata->length == 6);
@@ -168,7 +168,7 @@ additionaldata_eui48(ARGS_ADDLDATA) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 digest_eui48(ARGS_DIGEST) {
 	isc_region_t r;
 
@@ -180,7 +180,7 @@ digest_eui48(ARGS_DIGEST) {
 	return ((digest)(arg, &r));
 }
 
-static inline bool
+static bool
 checkowner_eui48(ARGS_CHECKOWNER) {
 	REQUIRE(type == dns_rdatatype_eui48);
 
@@ -192,7 +192,7 @@ checkowner_eui48(ARGS_CHECKOWNER) {
 	return (true);
 }
 
-static inline bool
+static bool
 checknames_eui48(ARGS_CHECKNAMES) {
 	REQUIRE(rdata->type == dns_rdatatype_eui48);
 	REQUIRE(rdata->length == 6);
@@ -204,7 +204,7 @@ checknames_eui48(ARGS_CHECKNAMES) {
 	return (true);
 }
 
-static inline int
+static int
 casecompare_eui48(ARGS_COMPARE) {
 	return (compare_eui48(rdata1, rdata2));
 }
