@@ -16,7 +16,7 @@
 
 #define RRTYPE_MF_ATTRIBUTES (0)
 
-static inline isc_result_t
+static isc_result_t
 fromtext_mf(ARGS_FROMTEXT) {
 	isc_token_t token;
 	dns_name_t name;
@@ -40,7 +40,7 @@ fromtext_mf(ARGS_FROMTEXT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 totext_mf(ARGS_TOTEXT) {
 	isc_region_t region;
 	dns_name_t name;
@@ -61,7 +61,7 @@ totext_mf(ARGS_TOTEXT) {
 	return (dns_name_totext(&prefix, sub, target));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromwire_mf(ARGS_FROMWIRE) {
 	dns_name_t name;
 
@@ -76,7 +76,7 @@ fromwire_mf(ARGS_FROMWIRE) {
 	return (dns_name_fromwire(&name, source, dctx, options, target));
 }
 
-static inline isc_result_t
+static isc_result_t
 towire_mf(ARGS_TOWIRE) {
 	dns_name_t name;
 	dns_offsets_t offsets;
@@ -94,7 +94,7 @@ towire_mf(ARGS_TOWIRE) {
 	return (dns_name_towire(&name, cctx, target));
 }
 
-static inline int
+static int
 compare_mf(ARGS_COMPARE) {
 	dns_name_t name1;
 	dns_name_t name2;
@@ -119,7 +119,7 @@ compare_mf(ARGS_COMPARE) {
 	return (dns_name_rdatacompare(&name1, &name2));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromstruct_mf(ARGS_FROMSTRUCT) {
 	dns_rdata_mf_t *mf = source;
 	isc_region_t region;
@@ -136,7 +136,7 @@ fromstruct_mf(ARGS_FROMSTRUCT) {
 	return (isc_buffer_copyregion(target, &region));
 }
 
-static inline isc_result_t
+static isc_result_t
 tostruct_mf(ARGS_TOSTRUCT) {
 	dns_rdata_mf_t *mf = target;
 	isc_region_t r;
@@ -159,7 +159,7 @@ tostruct_mf(ARGS_TOSTRUCT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline void
+static void
 freestruct_mf(ARGS_FREESTRUCT) {
 	dns_rdata_mf_t *mf = source;
 
@@ -173,7 +173,7 @@ freestruct_mf(ARGS_FREESTRUCT) {
 	mf->mctx = NULL;
 }
 
-static inline isc_result_t
+static isc_result_t
 additionaldata_mf(ARGS_ADDLDATA) {
 	dns_name_t name;
 	dns_offsets_t offsets;
@@ -190,7 +190,7 @@ additionaldata_mf(ARGS_ADDLDATA) {
 	return ((add)(arg, &name, dns_rdatatype_a, NULL));
 }
 
-static inline isc_result_t
+static isc_result_t
 digest_mf(ARGS_DIGEST) {
 	isc_region_t r;
 	dns_name_t name;
@@ -204,7 +204,7 @@ digest_mf(ARGS_DIGEST) {
 	return (dns_name_digest(&name, digest, arg));
 }
 
-static inline bool
+static bool
 checkowner_mf(ARGS_CHECKOWNER) {
 	REQUIRE(type == dns_rdatatype_mf);
 
@@ -216,7 +216,7 @@ checkowner_mf(ARGS_CHECKOWNER) {
 	return (true);
 }
 
-static inline bool
+static bool
 checknames_mf(ARGS_CHECKNAMES) {
 	REQUIRE(rdata->type == dns_rdatatype_mf);
 
@@ -227,7 +227,7 @@ checknames_mf(ARGS_CHECKNAMES) {
 	return (true);
 }
 
-static inline int
+static int
 casecompare_mf(ARGS_COMPARE) {
 	return (compare_mf(rdata1, rdata2));
 }
