@@ -44,7 +44,7 @@ check_mx(isc_token_t *token) {
 	return (true);
 }
 
-static inline isc_result_t
+static isc_result_t
 fromtext_mx(ARGS_FROMTEXT) {
 	isc_token_t token;
 	dns_name_t name;
@@ -96,7 +96,7 @@ fromtext_mx(ARGS_FROMTEXT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 totext_mx(ARGS_TOTEXT) {
 	isc_region_t region;
 	dns_name_t name;
@@ -124,7 +124,7 @@ totext_mx(ARGS_TOTEXT) {
 	return (dns_name_totext(&prefix, sub, target));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromwire_mx(ARGS_FROMWIRE) {
 	dns_name_t name;
 	isc_region_t sregion;
@@ -147,7 +147,7 @@ fromwire_mx(ARGS_FROMWIRE) {
 	return (dns_name_fromwire(&name, source, dctx, options, target));
 }
 
-static inline isc_result_t
+static isc_result_t
 towire_mx(ARGS_TOWIRE) {
 	dns_name_t name;
 	dns_offsets_t offsets;
@@ -168,7 +168,7 @@ towire_mx(ARGS_TOWIRE) {
 	return (dns_name_towire(&name, cctx, target));
 }
 
-static inline int
+static int
 compare_mx(ARGS_COMPARE) {
 	dns_name_t name1;
 	dns_name_t name2;
@@ -202,7 +202,7 @@ compare_mx(ARGS_COMPARE) {
 	return (dns_name_rdatacompare(&name1, &name2));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromstruct_mx(ARGS_FROMSTRUCT) {
 	dns_rdata_mx_t *mx = source;
 	isc_region_t region;
@@ -220,7 +220,7 @@ fromstruct_mx(ARGS_FROMSTRUCT) {
 	return (isc_buffer_copyregion(target, &region));
 }
 
-static inline isc_result_t
+static isc_result_t
 tostruct_mx(ARGS_TOSTRUCT) {
 	isc_region_t region;
 	dns_rdata_mx_t *mx = target;
@@ -245,7 +245,7 @@ tostruct_mx(ARGS_TOSTRUCT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline void
+static void
 freestruct_mx(ARGS_FREESTRUCT) {
 	dns_rdata_mx_t *mx = source;
 
@@ -265,7 +265,7 @@ static unsigned char port25_ndata[] = "\003_25\004_tcp";
 static dns_name_t port25 = DNS_NAME_INITNONABSOLUTE(port25_ndata,
 						    port25_offset);
 
-static inline isc_result_t
+static isc_result_t
 additionaldata_mx(ARGS_ADDLDATA) {
 	isc_result_t result;
 	dns_fixedname_t fixed;
@@ -302,7 +302,7 @@ additionaldata_mx(ARGS_ADDLDATA) {
 		      NULL));
 }
 
-static inline isc_result_t
+static isc_result_t
 digest_mx(ARGS_DIGEST) {
 	isc_region_t r1, r2;
 	dns_name_t name;
@@ -319,7 +319,7 @@ digest_mx(ARGS_DIGEST) {
 	return (dns_name_digest(&name, digest, arg));
 }
 
-static inline bool
+static bool
 checkowner_mx(ARGS_CHECKOWNER) {
 	REQUIRE(type == dns_rdatatype_mx);
 
@@ -329,7 +329,7 @@ checkowner_mx(ARGS_CHECKOWNER) {
 	return (dns_name_ishostname(name, wildcard));
 }
 
-static inline bool
+static bool
 checknames_mx(ARGS_CHECKNAMES) {
 	isc_region_t region;
 	dns_name_t name;
@@ -351,7 +351,7 @@ checknames_mx(ARGS_CHECKNAMES) {
 	return (true);
 }
 
-static inline int
+static int
 casecompare_mx(ARGS_COMPARE) {
 	return (compare_mx(rdata1, rdata2));
 }
