@@ -24,7 +24,7 @@
  * Check the wire format of the Regexp field.
  * Don't allow embedded NUL's.
  */
-static inline isc_result_t
+static isc_result_t
 txt_valid_regex(const unsigned char *txt) {
 	unsigned int nsub = 0;
 	char regex[256];
@@ -169,7 +169,7 @@ txt_valid_regex(const unsigned char *txt) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 fromtext_naptr(ARGS_FROMTEXT) {
 	isc_token_t token;
 	dns_name_t name;
@@ -239,7 +239,7 @@ fromtext_naptr(ARGS_FROMTEXT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 totext_naptr(ARGS_TOTEXT) {
 	isc_region_t region;
 	dns_name_t name;
@@ -300,7 +300,7 @@ totext_naptr(ARGS_TOTEXT) {
 	return (dns_name_totext(&prefix, sub, target));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromwire_naptr(ARGS_FROMWIRE) {
 	dns_name_t name;
 	isc_region_t sr;
@@ -348,7 +348,7 @@ fromwire_naptr(ARGS_FROMWIRE) {
 	return (dns_name_fromwire(&name, source, dctx, options, target));
 }
 
-static inline isc_result_t
+static isc_result_t
 towire_naptr(ARGS_TOWIRE) {
 	dns_name_t name;
 	dns_offsets_t offsets;
@@ -391,7 +391,7 @@ towire_naptr(ARGS_TOWIRE) {
 	return (dns_name_towire(&name, cctx, target));
 }
 
-static inline int
+static int
 compare_naptr(ARGS_COMPARE) {
 	dns_name_t name1;
 	dns_name_t name2;
@@ -463,7 +463,7 @@ compare_naptr(ARGS_COMPARE) {
 	return (dns_name_rdatacompare(&name1, &name2));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromstruct_naptr(ARGS_FROMSTRUCT) {
 	dns_rdata_naptr_t *naptr = source;
 	isc_region_t region;
@@ -491,7 +491,7 @@ fromstruct_naptr(ARGS_FROMSTRUCT) {
 	return (isc_buffer_copyregion(target, &region));
 }
 
-static inline isc_result_t
+static isc_result_t
 tostruct_naptr(ARGS_TOSTRUCT) {
 	dns_rdata_naptr_t *naptr = target;
 	isc_region_t r;
@@ -564,7 +564,7 @@ cleanup:
 	return (ISC_R_NOMEMORY);
 }
 
-static inline void
+static void
 freestruct_naptr(ARGS_FREESTRUCT) {
 	dns_rdata_naptr_t *naptr = source;
 
@@ -588,7 +588,7 @@ freestruct_naptr(ARGS_FREESTRUCT) {
 	naptr->mctx = NULL;
 }
 
-static inline isc_result_t
+static isc_result_t
 additionaldata_naptr(ARGS_ADDLDATA) {
 	dns_name_t name;
 	dns_offsets_t offsets;
@@ -648,7 +648,7 @@ additionaldata_naptr(ARGS_ADDLDATA) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 digest_naptr(ARGS_DIGEST) {
 	isc_region_t r1, r2;
 	unsigned int length, n;
@@ -707,7 +707,7 @@ digest_naptr(ARGS_DIGEST) {
 	return (dns_name_digest(&name, digest, arg));
 }
 
-static inline bool
+static bool
 checkowner_naptr(ARGS_CHECKOWNER) {
 	REQUIRE(type == dns_rdatatype_naptr);
 
@@ -719,7 +719,7 @@ checkowner_naptr(ARGS_CHECKOWNER) {
 	return (true);
 }
 
-static inline bool
+static bool
 checknames_naptr(ARGS_CHECKNAMES) {
 	REQUIRE(rdata->type == dns_rdatatype_naptr);
 
@@ -730,7 +730,7 @@ checknames_naptr(ARGS_CHECKNAMES) {
 	return (true);
 }
 
-static inline int
+static int
 casecompare_naptr(ARGS_COMPARE) {
 	return (compare_naptr(rdata1, rdata2));
 }
