@@ -16,14 +16,14 @@
 
 #define RRTYPE_NINFO_ATTRIBUTES (0)
 
-static inline isc_result_t
+static isc_result_t
 fromtext_ninfo(ARGS_FROMTEXT) {
 	REQUIRE(type == dns_rdatatype_ninfo);
 
 	return (generic_fromtext_txt(CALL_FROMTEXT));
 }
 
-static inline isc_result_t
+static isc_result_t
 totext_ninfo(ARGS_TOTEXT) {
 	REQUIRE(rdata != NULL);
 	REQUIRE(rdata->type == dns_rdatatype_ninfo);
@@ -31,14 +31,14 @@ totext_ninfo(ARGS_TOTEXT) {
 	return (generic_totext_txt(CALL_TOTEXT));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromwire_ninfo(ARGS_FROMWIRE) {
 	REQUIRE(type == dns_rdatatype_ninfo);
 
 	return (generic_fromwire_txt(CALL_FROMWIRE));
 }
 
-static inline isc_result_t
+static isc_result_t
 towire_ninfo(ARGS_TOWIRE) {
 	REQUIRE(rdata->type == dns_rdatatype_ninfo);
 
@@ -47,7 +47,7 @@ towire_ninfo(ARGS_TOWIRE) {
 	return (mem_tobuffer(target, rdata->data, rdata->length));
 }
 
-static inline int
+static int
 compare_ninfo(ARGS_COMPARE) {
 	isc_region_t r1;
 	isc_region_t r2;
@@ -61,14 +61,14 @@ compare_ninfo(ARGS_COMPARE) {
 	return (isc_region_compare(&r1, &r2));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromstruct_ninfo(ARGS_FROMSTRUCT) {
 	REQUIRE(type == dns_rdatatype_ninfo);
 
 	return (generic_fromstruct_txt(CALL_FROMSTRUCT));
 }
 
-static inline isc_result_t
+static isc_result_t
 tostruct_ninfo(ARGS_TOSTRUCT) {
 	dns_rdata_ninfo_t *ninfo = target;
 
@@ -82,7 +82,7 @@ tostruct_ninfo(ARGS_TOSTRUCT) {
 	return (generic_tostruct_txt(CALL_TOSTRUCT));
 }
 
-static inline void
+static void
 freestruct_ninfo(ARGS_FREESTRUCT) {
 	dns_rdata_ninfo_t *ninfo = source;
 
@@ -92,7 +92,7 @@ freestruct_ninfo(ARGS_FREESTRUCT) {
 	generic_freestruct_txt(source);
 }
 
-static inline isc_result_t
+static isc_result_t
 additionaldata_ninfo(ARGS_ADDLDATA) {
 	REQUIRE(rdata->type == dns_rdatatype_ninfo);
 
@@ -104,7 +104,7 @@ additionaldata_ninfo(ARGS_ADDLDATA) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 digest_ninfo(ARGS_DIGEST) {
 	isc_region_t r;
 
@@ -115,7 +115,7 @@ digest_ninfo(ARGS_DIGEST) {
 	return ((digest)(arg, &r));
 }
 
-static inline bool
+static bool
 checkowner_ninfo(ARGS_CHECKOWNER) {
 	REQUIRE(type == dns_rdatatype_ninfo);
 
@@ -127,7 +127,7 @@ checkowner_ninfo(ARGS_CHECKOWNER) {
 	return (true);
 }
 
-static inline bool
+static bool
 checknames_ninfo(ARGS_CHECKNAMES) {
 	REQUIRE(rdata->type == dns_rdatatype_ninfo);
 
@@ -138,7 +138,7 @@ checknames_ninfo(ARGS_CHECKNAMES) {
 	return (true);
 }
 
-static inline int
+static int
 casecompare_ninfo(ARGS_COMPARE) {
 	return (compare_ninfo(rdata1, rdata2));
 }
