@@ -18,7 +18,7 @@
 
 #define RRTYPE_RP_ATTRIBUTES (0)
 
-static inline isc_result_t
+static isc_result_t
 fromtext_rp(ARGS_FROMTEXT) {
 	isc_token_t token;
 	dns_name_t name;
@@ -57,7 +57,7 @@ fromtext_rp(ARGS_FROMTEXT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 totext_rp(ARGS_TOTEXT) {
 	isc_region_t region;
 	dns_name_t rmail;
@@ -89,7 +89,7 @@ totext_rp(ARGS_TOTEXT) {
 	return (dns_name_totext(&prefix, sub, target));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromwire_rp(ARGS_FROMWIRE) {
 	dns_name_t rmail;
 	dns_name_t email;
@@ -108,7 +108,7 @@ fromwire_rp(ARGS_FROMWIRE) {
 	return (dns_name_fromwire(&email, source, dctx, options, target));
 }
 
-static inline isc_result_t
+static isc_result_t
 towire_rp(ARGS_TOWIRE) {
 	isc_region_t region;
 	dns_name_t rmail;
@@ -136,7 +136,7 @@ towire_rp(ARGS_TOWIRE) {
 	return (dns_name_towire(&rmail, cctx, target));
 }
 
-static inline int
+static int
 compare_rp(ARGS_COMPARE) {
 	isc_region_t region1;
 	isc_region_t region2;
@@ -176,7 +176,7 @@ compare_rp(ARGS_COMPARE) {
 	return (dns_name_rdatacompare(&name1, &name2));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromstruct_rp(ARGS_FROMSTRUCT) {
 	dns_rdata_rp_t *rp = source;
 	isc_region_t region;
@@ -195,7 +195,7 @@ fromstruct_rp(ARGS_FROMSTRUCT) {
 	return (isc_buffer_copyregion(target, &region));
 }
 
-static inline isc_result_t
+static isc_result_t
 tostruct_rp(ARGS_TOSTRUCT) {
 	isc_region_t region;
 	dns_rdata_rp_t *rp = target;
@@ -222,7 +222,7 @@ tostruct_rp(ARGS_TOSTRUCT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline void
+static void
 freestruct_rp(ARGS_FREESTRUCT) {
 	dns_rdata_rp_t *rp = source;
 
@@ -238,7 +238,7 @@ freestruct_rp(ARGS_FREESTRUCT) {
 	rp->mctx = NULL;
 }
 
-static inline isc_result_t
+static isc_result_t
 additionaldata_rp(ARGS_ADDLDATA) {
 	REQUIRE(rdata->type == dns_rdatatype_rp);
 
@@ -250,7 +250,7 @@ additionaldata_rp(ARGS_ADDLDATA) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 digest_rp(ARGS_DIGEST) {
 	isc_region_t r;
 	dns_name_t name;
@@ -270,7 +270,7 @@ digest_rp(ARGS_DIGEST) {
 	return (dns_name_digest(&name, digest, arg));
 }
 
-static inline bool
+static bool
 checkowner_rp(ARGS_CHECKOWNER) {
 	REQUIRE(type == dns_rdatatype_rp);
 
@@ -282,7 +282,7 @@ checkowner_rp(ARGS_CHECKOWNER) {
 	return (true);
 }
 
-static inline bool
+static bool
 checknames_rp(ARGS_CHECKNAMES) {
 	isc_region_t region;
 	dns_name_t name;
@@ -303,7 +303,7 @@ checknames_rp(ARGS_CHECKNAMES) {
 	return (true);
 }
 
-static inline int
+static int
 casecompare_rp(ARGS_COMPARE) {
 	return (compare_rp(rdata1, rdata2));
 }
