@@ -12,8 +12,7 @@
  */
 
 /* */
-#ifndef GENERIC_SOA_6_H
-#define GENERIC_SOA_6_H 1
+#pragma once
 
 typedef struct dns_rdata_soa {
 	dns_rdatacommon_t common;
@@ -26,5 +25,3 @@ typedef struct dns_rdata_soa {
 	uint32_t expire;  /*%< host order */
 	uint32_t minimum; /*%< host order */
 } dns_rdata_soa_t;
-
-#endif /* GENERIC_SOA_6_H */
