@@ -18,7 +18,7 @@
 
 #define RRTYPE_TKEY_ATTRIBUTES (DNS_RDATATYPEATTR_META)
 
-static inline isc_result_t
+static isc_result_t
 fromtext_tkey(ARGS_FROMTEXT) {
 	isc_token_t token;
 	dns_rcode_t rcode;
@@ -75,7 +75,8 @@ fromtext_tkey(ARGS_FROMTEXT) {
 	RETERR(isc_lex_getmastertoken(lexer, &token, isc_tokentype_string,
 				      false));
 	if (dns_tsigrcode_fromtext(&rcode, &token.value.as_textregion) !=
-	    ISC_R_SUCCESS) {
+	    ISC_R_SUCCESS)
+	{
 		i = strtol(DNS_AS_STR(token), &e, 10);
 		if (*e != 0) {
 			RETTOK(DNS_R_UNKNOWN);
@@ -118,7 +119,7 @@ fromtext_tkey(ARGS_FROMTEXT) {
 	return (isc_base64_tobuffer(lexer, target, (int)token.value.as_ulong));
 }
 
-static inline isc_result_t
+static isc_result_t
 totext_tkey(ARGS_TOTEXT) {
 	isc_region_t sr, dr;
 	char buf[sizeof("4294967295 ")];
@@ -242,7 +243,7 @@ totext_tkey(ARGS_TOTEXT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 fromwire_tkey(ARGS_FROMWIRE) {
 	isc_region_t sr;
 	unsigned long n;
@@ -303,7 +304,7 @@ fromwire_tkey(ARGS_FROMWIRE) {
 	return (mem_tobuffer(target, sr.base, n + 2));
 }
 
-static inline isc_result_t
+static isc_result_t
 towire_tkey(ARGS_TOWIRE) {
 	isc_region_t sr;
 	dns_name_t name;
@@ -325,7 +326,7 @@ towire_tkey(ARGS_TOWIRE) {
 	return (mem_tobuffer(target, sr.base, sr.length));
 }
 
-static inline int
+static int
 compare_tkey(ARGS_COMPARE) {
 	isc_region_t r1;
 	isc_region_t r2;
@@ -356,7 +357,7 @@ compare_tkey(ARGS_COMPARE) {
 	return (isc_region_compare(&r1, &r2));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromstruct_tkey(ARGS_FROMSTRUCT) {
 	dns_rdata_tkey_t *tkey = source;
 
@@ -414,7 +415,7 @@ fromstruct_tkey(ARGS_FROMSTRUCT) {
 	return (mem_tobuffer(target, tkey->other, tkey->otherlen));
 }
 
-static inline isc_result_t
+static isc_result_t
 tostruct_tkey(ARGS_TOSTRUCT) {
 	dns_rdata_tkey_t *tkey = target;
 	dns_name_t alg;
@@ -507,7 +508,7 @@ cleanup:
 	return (ISC_R_NOMEMORY);
 }
 
-static inline void
+static void
 freestruct_tkey(ARGS_FREESTRUCT) {
 	dns_rdata_tkey_t *tkey = (dns_rdata_tkey_t *)source;
 
@@ -527,7 +528,7 @@ freestruct_tkey(ARGS_FREESTRUCT) {
 	tkey->mctx = NULL;
 }
 
-static inline isc_result_t
+static isc_result_t
 additionaldata_tkey(ARGS_ADDLDATA) {
 	REQUIRE(rdata->type == dns_rdatatype_tkey);
 
@@ -539,7 +540,7 @@ additionaldata_tkey(ARGS_ADDLDATA) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 digest_tkey(ARGS_DIGEST) {
 	UNUSED(rdata);
 	UNUSED(digest);
@@ -550,7 +551,7 @@ digest_tkey(ARGS_DIGEST) {
 	return (ISC_R_NOTIMPLEMENTED);
 }
 
-static inline bool
+static bool
 checkowner_tkey(ARGS_CHECKOWNER) {
 	REQUIRE(type == dns_rdatatype_tkey);
 
@@ -562,7 +563,7 @@ checkowner_tkey(ARGS_CHECKOWNER) {
 	return (true);
 }
 
-static inline bool
+static bool
 checknames_tkey(ARGS_CHECKNAMES) {
 	REQUIRE(rdata->type == dns_rdatatype_tkey);
 
@@ -573,7 +574,7 @@ checknames_tkey(ARGS_CHECKNAMES) {
 	return (true);
 }
 
-static inline int
+static int
 casecompare_tkey(ARGS_COMPARE) {
 	return (compare_tkey(rdata1, rdata2));
 }
