@@ -18,7 +18,7 @@
 
 #define RRTYPE_TLSA_ATTRIBUTES 0
 
-static inline isc_result_t
+static isc_result_t
 generic_fromtext_tlsa(ARGS_FROMTEXT) {
 	isc_token_t token;
 
@@ -64,7 +64,7 @@ generic_fromtext_tlsa(ARGS_FROMTEXT) {
 	return (isc_hex_tobuffer(lexer, target, -2));
 }
 
-static inline isc_result_t
+static isc_result_t
 generic_totext_tlsa(ARGS_TOTEXT) {
 	isc_region_t sr;
 	char buf[sizeof("64000 ")];
@@ -119,7 +119,7 @@ generic_totext_tlsa(ARGS_TOTEXT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 generic_fromwire_tlsa(ARGS_FROMWIRE) {
 	isc_region_t sr;
 
@@ -139,28 +139,28 @@ generic_fromwire_tlsa(ARGS_FROMWIRE) {
 	return (mem_tobuffer(target, sr.base, sr.length));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromtext_tlsa(ARGS_FROMTEXT) {
 	REQUIRE(type == dns_rdatatype_tlsa);
 
 	return (generic_fromtext_tlsa(CALL_FROMTEXT));
 }
 
-static inline isc_result_t
+static isc_result_t
 totext_tlsa(ARGS_TOTEXT) {
 	REQUIRE(rdata->type == dns_rdatatype_tlsa);
 
 	return (generic_totext_tlsa(CALL_TOTEXT));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromwire_tlsa(ARGS_FROMWIRE) {
 	REQUIRE(type == dns_rdatatype_tlsa);
 
 	return (generic_fromwire_tlsa(CALL_FROMWIRE));
 }
 
-static inline isc_result_t
+static isc_result_t
 towire_tlsa(ARGS_TOWIRE) {
 	isc_region_t sr;
 
@@ -173,7 +173,7 @@ towire_tlsa(ARGS_TOWIRE) {
 	return (mem_tobuffer(target, sr.base, sr.length));
 }
 
-static inline int
+static int
 compare_tlsa(ARGS_COMPARE) {
 	isc_region_t r1;
 	isc_region_t r2;
@@ -189,7 +189,7 @@ compare_tlsa(ARGS_COMPARE) {
 	return (isc_region_compare(&r1, &r2));
 }
 
-static inline isc_result_t
+static isc_result_t
 generic_fromstruct_tlsa(ARGS_FROMSTRUCT) {
 	dns_rdata_tlsa_t *tlsa = source;
 
@@ -207,7 +207,7 @@ generic_fromstruct_tlsa(ARGS_FROMSTRUCT) {
 	return (mem_tobuffer(target, tlsa->data, tlsa->length));
 }
 
-static inline isc_result_t
+static isc_result_t
 generic_tostruct_tlsa(ARGS_TOSTRUCT) {
 	dns_rdata_tlsa_t *tlsa = target;
 	isc_region_t region;
@@ -239,7 +239,7 @@ generic_tostruct_tlsa(ARGS_TOSTRUCT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline void
+static void
 generic_freestruct_tlsa(ARGS_FREESTRUCT) {
 	dns_rdata_tlsa_t *tlsa = source;
 
@@ -255,14 +255,14 @@ generic_freestruct_tlsa(ARGS_FREESTRUCT) {
 	tlsa->mctx = NULL;
 }
 
-static inline isc_result_t
+static isc_result_t
 fromstruct_tlsa(ARGS_FROMSTRUCT) {
 	REQUIRE(type == dns_rdatatype_tlsa);
 
 	return (generic_fromstruct_tlsa(CALL_FROMSTRUCT));
 }
 
-static inline isc_result_t
+static isc_result_t
 tostruct_tlsa(ARGS_TOSTRUCT) {
 	dns_rdata_tlsa_t *tlsa = target;
 
@@ -276,7 +276,7 @@ tostruct_tlsa(ARGS_TOSTRUCT) {
 	return (generic_tostruct_tlsa(CALL_TOSTRUCT));
 }
 
-static inline void
+static void
 freestruct_tlsa(ARGS_FREESTRUCT) {
 	dns_rdata_tlsa_t *tlsa = source;
 
@@ -286,7 +286,7 @@ freestruct_tlsa(ARGS_FREESTRUCT) {
 	generic_freestruct_tlsa(source);
 }
 
-static inline isc_result_t
+static isc_result_t
 additionaldata_tlsa(ARGS_ADDLDATA) {
 	REQUIRE(rdata->type == dns_rdatatype_tlsa);
 
@@ -298,7 +298,7 @@ additionaldata_tlsa(ARGS_ADDLDATA) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 digest_tlsa(ARGS_DIGEST) {
 	isc_region_t r;
 
@@ -309,7 +309,7 @@ digest_tlsa(ARGS_DIGEST) {
 	return ((digest)(arg, &r));
 }
 
-static inline bool
+static bool
 checkowner_tlsa(ARGS_CHECKOWNER) {
 	REQUIRE(type == dns_rdatatype_tlsa);
 
@@ -321,7 +321,7 @@ checkowner_tlsa(ARGS_CHECKOWNER) {
 	return (true);
 }
 
-static inline bool
+static bool
 checknames_tlsa(ARGS_CHECKNAMES) {
 	REQUIRE(rdata->type == dns_rdatatype_tlsa);
 
@@ -332,7 +332,7 @@ checknames_tlsa(ARGS_CHECKNAMES) {
 	return (true);
 }
 
-static inline int
+static int
 casecompare_tlsa(ARGS_COMPARE) {
 	return (compare_tlsa(rdata1, rdata2));
 }
