@@ -16,7 +16,7 @@
 
 #define RRTYPE_TXT_ATTRIBUTES (0)
 
-static inline isc_result_t
+static isc_result_t
 generic_fromtext_txt(ARGS_FROMTEXT) {
 	isc_token_t token;
 	int strings;
@@ -39,7 +39,8 @@ generic_fromtext_txt(ARGS_FROMTEXT) {
 		RETERR(isc_lex_getmastertoken(lexer, &token,
 					      isc_tokentype_qstring, true));
 		if (token.type != isc_tokentype_qstring &&
-		    token.type != isc_tokentype_string) {
+		    token.type != isc_tokentype_string)
+		{
 			break;
 		}
 		RETTOK(txt_fromtext(&token.value.as_textregion, target));
@@ -50,7 +51,7 @@ generic_fromtext_txt(ARGS_FROMTEXT) {
 	return (strings == 0 ? ISC_R_UNEXPECTEDEND : ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 generic_totext_txt(ARGS_TOTEXT) {
 	isc_region_t region;
 
@@ -68,7 +69,7 @@ generic_totext_txt(ARGS_TOTEXT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 generic_fromwire_txt(ARGS_FROMWIRE) {
 	isc_result_t result;
 
@@ -86,14 +87,14 @@ generic_fromwire_txt(ARGS_FROMWIRE) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 fromtext_txt(ARGS_FROMTEXT) {
 	REQUIRE(type == dns_rdatatype_txt);
 
 	return (generic_fromtext_txt(CALL_FROMTEXT));
 }
 
-static inline isc_result_t
+static isc_result_t
 totext_txt(ARGS_TOTEXT) {
 	REQUIRE(rdata != NULL);
 	REQUIRE(rdata->type == dns_rdatatype_txt);
@@ -101,14 +102,14 @@ totext_txt(ARGS_TOTEXT) {
 	return (generic_totext_txt(CALL_TOTEXT));
 }
 
-static inline isc_result_t
+static isc_result_t
 fromwire_txt(ARGS_FROMWIRE) {
 	REQUIRE(type == dns_rdatatype_txt);
 
 	return (generic_fromwire_txt(CALL_FROMWIRE));
 }
 
-static inline isc_result_t
+static isc_result_t
 towire_txt(ARGS_TOWIRE) {
 	REQUIRE(rdata->type == dns_rdatatype_txt);
 
@@ -117,7 +118,7 @@ towire_txt(ARGS_TOWIRE) {
 	return (mem_tobuffer(target, rdata->data, rdata->length));
 }
 
-static inline int
+static int
 compare_txt(ARGS_COMPARE) {
 	isc_region_t r1;
 	isc_region_t r2;
@@ -131,7 +132,7 @@ compare_txt(ARGS_COMPARE) {
 	return (isc_region_compare(&r1, &r2));
 }
 
-static inline isc_result_t
+static isc_result_t
 generic_fromstruct_txt(ARGS_FROMSTRUCT) {
 	dns_rdata_txt_t *txt = source;
 	isc_region_t region;
@@ -159,7 +160,7 @@ generic_fromstruct_txt(ARGS_FROMSTRUCT) {
 	return (mem_tobuffer(target, txt->txt, txt->txt_len));
 }
 
-static inline isc_result_t
+static isc_result_t
 generic_tostruct_txt(ARGS_TOSTRUCT) {
 	dns_rdata_txt_t *txt = target;
 	isc_region_t r;
@@ -181,7 +182,7 @@ generic_tostruct_txt(ARGS_TOSTRUCT) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline void
+static void
 generic_freestruct_txt(ARGS_FREESTRUCT) {
 	dns_rdata_txt_t *txt = source;
 
@@ -197,14 +198,14 @@ generic_freestruct_txt(ARGS_FREESTRUCT) {
 	txt->mctx = NULL;
 }
 
-static inline isc_result_t
+static isc_result_t
 fromstruct_txt(ARGS_FROMSTRUCT) {
 	REQUIRE(type == dns_rdatatype_txt);
 
 	return (generic_fromstruct_txt(CALL_FROMSTRUCT));
 }
 
-static inline isc_result_t
+static isc_result_t
 tostruct_txt(ARGS_TOSTRUCT) {
 	dns_rdata_txt_t *txt = target;
 
@@ -218,7 +219,7 @@ tostruct_txt(ARGS_TOSTRUCT) {
 	return (generic_tostruct_txt(CALL_TOSTRUCT));
 }
 
-static inline void
+static void
 freestruct_txt(ARGS_FREESTRUCT) {
 	dns_rdata_txt_t *txt = source;
 
@@ -228,7 +229,7 @@ freestruct_txt(ARGS_FREESTRUCT) {
 	generic_freestruct_txt(source);
 }
 
-static inline isc_result_t
+static isc_result_t
 additionaldata_txt(ARGS_ADDLDATA) {
 	REQUIRE(rdata->type == dns_rdatatype_txt);
 
@@ -240,7 +241,7 @@ additionaldata_txt(ARGS_ADDLDATA) {
 	return (ISC_R_SUCCESS);
 }
 
-static inline isc_result_t
+static isc_result_t
 digest_txt(ARGS_DIGEST) {
 	isc_region_t r;
 
@@ -251,7 +252,7 @@ digest_txt(ARGS_DIGEST) {
 	return ((digest)(arg, &r));
 }
 
-static inline bool
+static bool
 checkowner_txt(ARGS_CHECKOWNER) {
 	REQUIRE(type == dns_rdatatype_txt);
 
@@ -263,7 +264,7 @@ checkowner_txt(ARGS_CHECKOWNER) {
 	return (true);
 }
 
-static inline bool
+static bool
 checknames_txt(ARGS_CHECKNAMES) {
 	REQUIRE(rdata->type == dns_rdatatype_txt);
 
@@ -274,7 +275,7 @@ checknames_txt(ARGS_CHECKNAMES) {
 	return (true);
 }
 
-static inline int
+static int
 casecompare_txt(ARGS_COMPARE) {
 	return (compare_txt(rdata1, rdata2));
 }
