@@ -1,28 +1,29 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
+ * SPDX-License-Identifier: MPL-2.0
+ *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * file, you can obtain one at https://mozilla.org/MPL/2.0/.
  *
  * See the COPYRIGHT file distributed with this work for additional
  * information regarding copyright ownership.
  */
 
 /* */
-#ifndef GENERIC_TXT_16_H
-#define GENERIC_TXT_16_H 1
+#pragma once
 
 typedef struct dns_rdata_txt_string {
-	uint8_t	       length;
+	uint8_t length;
 	unsigned char *data;
 } dns_rdata_txt_string_t;
 
 typedef struct dns_rdata_txt {
 	dns_rdatacommon_t common;
-	isc_mem_t *	  mctx;
-	unsigned char *	  txt;
-	uint16_t	  txt_len;
+	isc_mem_t *mctx;
+	unsigned char *txt;
+	uint16_t txt_len;
 	/* private */
 	uint16_t offset;
 } dns_rdata_txt_t;
@@ -40,5 +41,3 @@ dns_rdata_txt_next(dns_rdata_txt_t *);
 
 isc_result_t
 dns_rdata_txt_current(dns_rdata_txt_t *, dns_rdata_txt_string_t *);
-
-#endif /* GENERIC_TXT_16_H */
