@@ -1,9 +1,11 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
+ * SPDX-License-Identifier: MPL-2.0
+ *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * file, you can obtain one at https://mozilla.org/MPL/2.0/.
  *
  * See the COPYRIGHT file distributed with this work for additional
  * information regarding copyright ownership.
@@ -22,17 +24,17 @@
 #include <isc/parseint.h>
 #include <isc/print.h>
 #include <isc/region.h>
+#include <isc/result.h>
 #include <isc/string.h>
 #include <isc/util.h>
 
-#include <dns/result.h>
 #include <dns/ttl.h>
 
 #define RETERR(x)                        \
 	do {                             \
 		isc_result_t _r = (x);   \
 		if (_r != ISC_R_SUCCESS) \
-			return (_r);     \
+			return ((_r));   \
 	} while (0)
 
 static isc_result_t
@@ -43,22 +45,23 @@ bind_ttl(isc_textregion_t *source, uint32_t *ttl);
  */
 static isc_result_t
 ttlfmt(unsigned int t, const char *s, bool verbose, bool space,
-       isc_buffer_t *target)
-{
-	char	     tmp[60];
+       isc_buffer_t *target) {
+	char tmp[60];
 	unsigned int len;
 	isc_region_t region;
 
-	if (verbose)
+	if (verbose) {
 		len = snprintf(tmp, sizeof(tmp), "%s%u %s%s", space ? " " : "",
 			       t, s, t == 1 ? "" : "s");
-	else
+	} else {
 		len = snprintf(tmp, sizeof(tmp), "%u%c", t, s[0]);
+	}
 
 	INSIST(len + 1 <= sizeof(tmp));
 	isc_buffer_availableregion(target, &region);
-	if (len > region.length)
+	if (len > region.length) {
 		return (ISC_R_NOSPACE);
+	}
 	memmove(region.base, tmp, len);
 	isc_buffer_add(target, len);
 
@@ -69,8 +72,7 @@ ttlfmt(unsigned int t, const char *s, bool verbose, bool space,
  * Derived from bind8 ns_format_ttl().
  */
 isc_result_t
-dns_ttl_totext(uint32_t src, bool verbose, bool upcase, isc_buffer_t *target)
-{
+dns_ttl_totext(uint32_t src, bool verbose, bool upcase, isc_buffer_t *target) {
 	unsigned secs, mins, hours, days, weeks, x;
 
 	secs = src % 60;
@@ -129,37 +131,36 @@ dns_ttl_totext(uint32_t src, bool verbose, bool upcase, isc_buffer_t *target)
 }
 
 isc_result_t
-dns_counter_fromtext(isc_textregion_t *source, uint32_t *ttl)
-{
+dns_counter_fromtext(isc_textregion_t *source, uint32_t *ttl) {
 	return (bind_ttl(source, ttl));
 }
 
 isc_result_t
-dns_ttl_fromtext(isc_textregion_t *source, uint32_t *ttl)
-{
+dns_ttl_fromtext(isc_textregion_t *source, uint32_t *ttl) {
 	isc_result_t result;
 
 	result = bind_ttl(source, ttl);
-	if (result != ISC_R_SUCCESS && result != ISC_R_RANGE)
+	if (result != ISC_R_SUCCESS && result != ISC_R_RANGE) {
 		result = DNS_R_BADTTL;
+	}
 	return (result);
 }
 
 static isc_result_t
-bind_ttl(isc_textregion_t *source, uint32_t *ttl)
-{
+bind_ttl(isc_textregion_t *source, uint32_t *ttl) {
 	uint64_t tmp = 0ULL;
 	uint32_t n;
-	char *	 s;
-	char	 buf[64];
-	char	 nbuf[64]; /* Number buffer */
+	char *s;
+	char buf[64];
+	char nbuf[64]; /* Number buffer */
 
 	/*
 	 * Copy the buffer as it may not be NULL terminated.
 	 * No legal counter / ttl is longer that 63 characters.
 	 */
-	if (source->length > sizeof(buf) - 1)
+	if (source->length > sizeof(buf) - 1) {
 		return (DNS_R_SYNTAX);
+	}
 	/* Copy source->length bytes and NUL terminate. */
 	snprintf(buf, sizeof(buf), "%.*s", (int)source->length, source->base);
 	s = buf;
@@ -168,13 +169,15 @@ bind_ttl(isc_textregion_t *source, uint32_t *ttl)
 		isc_result_t result;
 
 		char *np = nbuf;
-		while (*s != '\0' && isdigit((unsigned char)*s))
+		while (*s != '\0' && isdigit((unsigned char)*s)) {
 			*np++ = *s++;
+		}
 		*np++ = '\0';
 		INSIST(np - nbuf <= (int)sizeof(nbuf));
 		result = isc_parse_uint32(&n, nbuf, 10);
-		if (result != ISC_R_SUCCESS)
+		if (result != ISC_R_SUCCESS) {
 			return (DNS_R_SYNTAX);
+		}
 		switch (*s) {
 		case 'w':
 		case 'W':
@@ -203,8 +206,9 @@ bind_ttl(isc_textregion_t *source, uint32_t *ttl)
 			break;
 		case '\0':
 			/* Plain number? */
-			if (tmp != 0ULL)
+			if (tmp != 0ULL) {
 				return (DNS_R_SYNTAX);
+			}
 			tmp = n;
 			break;
 		default:
@@ -212,8 +216,9 @@ bind_ttl(isc_textregion_t *source, uint32_t *ttl)
 		}
 	} while (*s != '\0');
 
-	if (tmp > 0xffffffffULL)
+	if (tmp > 0xffffffffULL) {
 		return (ISC_R_RANGE);
+	}
 
 	*ttl = (uint32_t)(tmp & 0xffffffffUL);
 	return (ISC_R_SUCCESS);
