@@ -22,7 +22,6 @@
 #include <stdlib.h>
 
 #include <isc/align.h>
-#include <isc/bind9.h>
 #include <isc/hash.h>
 #include <isc/magic.h>
 #include <isc/mem.h>
@@ -31,7 +30,6 @@
 #include <isc/os.h>
 #include <isc/print.h>
 #include <isc/refcount.h>
-#include <isc/strerr.h>
 #include <isc/string.h>
 #include <isc/types.h>
 #include <isc/util.h>
@@ -45,13 +43,17 @@
 #include <json_object.h>
 #endif /* HAVE_JSON_C */
 
-#if defined(HAVE_MALLOC_NP_H)
+/* On DragonFly BSD the header does not provide jemalloc API */
+#if defined(HAVE_MALLOC_NP_H) && !defined(__DragonFly__)
 #include <malloc_np.h>
+#define JEMALLOC_API_SUPPORTED 1
 #elif defined(HAVE_JEMALLOC)
 #include <jemalloc/jemalloc.h>
+#define JEMALLOC_API_SUPPORTED 1
 
 #if JEMALLOC_VERSION_MAJOR < 4
 #define sdallocx(ptr, size, flags) dallocx(ptr, flags)
+#define MALLOCX_TCACHE_NONE	   (0)
 #endif /* JEMALLOC_VERSION_MAJOR < 4 */
 
 #else
@@ -69,6 +71,10 @@
 unsigned int isc_mem_debugging = ISC_MEM_DEBUGGING;
 unsigned int isc_mem_defaultflags = ISC_MEMFLAG_DEFAULT;
 
+#define ISC_MEM_ILLEGAL_ARENA (UINT_MAX)
+
+volatile void *isc__mem_malloc = mallocx;
+
 /*
  * Constants.
  */
@@ -132,6 +138,8 @@ static uint64_t totallost;
 struct isc_mem {
 	unsigned int magic;
 	unsigned int flags;
+	unsigned int jemalloc_flags;
+	unsigned int jemalloc_arena;
 	isc_mutex_t lock;
 	bool checkfree;
 	struct stats stats[STATS_BUCKETS + 1];
@@ -207,7 +215,7 @@ static void
 print_active(isc_mem_t *ctx, FILE *out);
 #endif /* ISC_MEM_TRACKLINES */
 
-static inline size_t
+static size_t
 increment_malloced(isc_mem_t *ctx, size_t size) {
 	size_t malloced = atomic_fetch_add_relaxed(&ctx->malloced, size) + size;
 	size_t maxmalloced = atomic_load_relaxed(&ctx->maxmalloced);
@@ -220,7 +228,7 @@ increment_malloced(isc_mem_t *ctx, size_t size) {
 	return (malloced);
 }
 
-static inline size_t
+static size_t
 decrement_malloced(isc_mem_t *ctx, size_t size) {
 	size_t malloced = atomic_fetch_sub_relaxed(&ctx->malloced, size) - size;
 
@@ -259,9 +267,9 @@ add_trace_entry(isc_mem_t *mctx, const void *ptr, size_t size FLARG) {
 #endif
 	idx = hash % DEBUG_TABLE_COUNT;
 
-	dl = mallocx(sizeof(debuglink_t), 0);
+	dl = mallocx(sizeof(*dl), mctx->jemalloc_flags);
 	INSIST(dl != NULL);
-	increment_malloced(mctx, sizeof(debuglink_t));
+	increment_malloced(mctx, sizeof(*dl));
 
 	ISC_LINK_INIT(dl, link);
 	dl->ptr = ptr;
@@ -309,7 +317,7 @@ delete_trace_entry(isc_mem_t *mctx, const void *ptr, size_t size,
 		if (dl->ptr == ptr) {
 			ISC_LIST_UNLINK(mctx->debuglist[idx], dl, link);
 			decrement_malloced(mctx, sizeof(*dl));
-			sdallocx(dl, sizeof(*dl), 0);
+			sdallocx(dl, sizeof(*dl), mctx->jemalloc_flags);
 			goto unlock;
 		}
 		dl = ISC_LIST_NEXT(dl, link);
@@ -319,8 +327,7 @@ delete_trace_entry(isc_mem_t *mctx, const void *ptr, size_t size,
 	 * If we get here, we didn't find the item on the list.  We're
 	 * screwed.
 	 */
-	INSIST(0);
-	ISC_UNREACHABLE();
+	UNREACHABLE();
 unlock:
 	MCTXUNLOCK(mctx);
 }
@@ -336,13 +343,13 @@ unlock:
 /*!
  * Perform a malloc, doing memory filling and overrun detection as necessary.
  */
-static inline void *
+static void *
 mem_get(isc_mem_t *ctx, size_t size, int flags) {
 	char *ret = NULL;
 
 	ADJUST_ZERO_ALLOCATION_SIZE(size);
 
-	ret = mallocx(size, flags);
+	ret = mallocx(size, flags | ctx->jemalloc_flags);
 	INSIST(ret != NULL);
 
 	if ((ctx->flags & ISC_MEMFLAG_FILL) != 0) {
@@ -356,24 +363,24 @@ mem_get(isc_mem_t *ctx, size_t size, int flags) {
  * Perform a free, doing memory filling and overrun detection as necessary.
  */
 /* coverity[+free : arg-1] */
-static inline void
+static void
 mem_put(isc_mem_t *ctx, void *mem, size_t size, int flags) {
 	ADJUST_ZERO_ALLOCATION_SIZE(size);
 
 	if ((ctx->flags & ISC_MEMFLAG_FILL) != 0) {
 		memset(mem, 0xde, size); /* Mnemonic for "dead". */
 	}
-	sdallocx(mem, size, flags);
+	sdallocx(mem, size, flags | ctx->jemalloc_flags);
 }
 
-static inline void *
+static void *
 mem_realloc(isc_mem_t *ctx, void *old_ptr, size_t old_size, size_t new_size,
 	    int flags) {
 	void *new_ptr = NULL;
 
 	ADJUST_ZERO_ALLOCATION_SIZE(new_size);
 
-	new_ptr = rallocx(old_ptr, new_size, flags);
+	new_ptr = rallocx(old_ptr, new_size, flags | ctx->jemalloc_flags);
 	INSIST(new_ptr != NULL);
 
 	if ((ctx->flags & ISC_MEMFLAG_FILL) != 0) {
@@ -396,7 +403,7 @@ mem_realloc(isc_mem_t *ctx, void *old_ptr, size_t old_size, size_t new_size,
 /*!
  * Update internal counters after a memory get.
  */
-static inline void
+static void
 mem_getstats(isc_mem_t *ctx, size_t size) {
 	struct stats *stats = stats_bucket(ctx, size);
 
@@ -412,15 +419,18 @@ mem_getstats(isc_mem_t *ctx, size_t size) {
 /*!
  * Update internal counters after a memory put.
  */
-static inline void
+static void
 mem_putstats(isc_mem_t *ctx, void *ptr, size_t size) {
 	struct stats *stats = stats_bucket(ctx, size);
+	atomic_size_t s, g;
 
 	UNUSED(ptr);
 
-	INSIST(atomic_fetch_sub_release(&ctx->inuse, size) >= size);
+	s = atomic_fetch_sub_release(&ctx->inuse, size);
+	INSIST(s >= size);
 
-	INSIST(atomic_fetch_sub_release(&stats->gets, 1) >= 1);
+	g = atomic_fetch_sub_release(&stats->gets, 1);
+	INSIST(g >= 1);
 
 	decrement_malloced(ctx, size);
 }
@@ -429,6 +439,48 @@ mem_putstats(isc_mem_t *ctx, void *ptr, size_t size) {
  * Private.
  */
 
+static bool
+mem_jemalloc_arena_create(unsigned int *pnew_arenano) {
+	REQUIRE(pnew_arenano != NULL);
+
+#if defined(JEMALLOC_API_SUPPORTED) && JEMALLOC_VERSION_MAJOR >= 4
+	unsigned int arenano = 0;
+	size_t len = sizeof(arenano);
+	int res = 0;
+
+	res = mallctl("arenas.create", &arenano, &len, NULL, 0);
+	if (res != 0) {
+		return (false);
+	}
+
+	*pnew_arenano = arenano;
+
+	return (true);
+#else
+	*pnew_arenano = ISC_MEM_ILLEGAL_ARENA;
+	return (true);
+#endif /* defined(JEMALLOC_API_SUPPORTED) && JEMALLOC_VERSION_MAJOR >= 4 */
+}
+
+static bool
+mem_jemalloc_arena_destroy(unsigned int arenano) {
+#if defined(JEMALLOC_API_SUPPORTED) && JEMALLOC_VERSION_MAJOR >= 4
+	int res = 0;
+	char buf[256] = { 0 };
+
+	(void)snprintf(buf, sizeof(buf), "arena.%u.destroy", arenano);
+	res = mallctl(buf, NULL, NULL, NULL, 0);
+	if (res != 0) {
+		return (false);
+	}
+
+	return (true);
+#else
+	UNUSED(arenano);
+	return (true);
+#endif /* defined(JEMALLOC_API_SUPPORTED) && JEMALLOC_VERSION_MAJOR >= 4 */
+}
+
 static void
 mem_initialize(void) {
 	isc_mutex_init(&contextslock);
@@ -454,17 +506,20 @@ isc__mem_shutdown(void) {
 }
 
 static void
-mem_create(isc_mem_t **ctxp, unsigned int flags) {
+mem_create(isc_mem_t **ctxp, unsigned int flags, unsigned int jemalloc_flags) {
 	isc_mem_t *ctx = NULL;
 
 	REQUIRE(ctxp != NULL && *ctxp == NULL);
 
-	ctx = mallocx(sizeof(*ctx), MALLOCX_ALIGN(isc_os_cacheline()));
+	ctx = mallocx(sizeof(*ctx),
+		      MALLOCX_ALIGN(isc_os_cacheline()) | jemalloc_flags);
 	INSIST(ctx != NULL);
 
 	*ctx = (isc_mem_t){
 		.magic = MEM_MAGIC,
 		.flags = flags,
+		.jemalloc_flags = jemalloc_flags,
+		.jemalloc_arena = ISC_MEM_ILLEGAL_ARENA,
 		.checkfree = true,
 	};
 
@@ -492,7 +547,8 @@ mem_create(isc_mem_t **ctxp, unsigned int flags) {
 		unsigned int i;
 
 		ctx->debuglist =
-			mallocx((DEBUG_TABLE_COUNT * sizeof(debuglist_t)), 0);
+			mallocx((DEBUG_TABLE_COUNT * sizeof(debuglist_t)),
+				ctx->jemalloc_flags);
 		INSIST(ctx->debuglist != NULL);
 
 		for (i = 0; i < DEBUG_TABLE_COUNT; i++) {
@@ -518,6 +574,7 @@ static void
 destroy(isc_mem_t *ctx) {
 	unsigned int i;
 	size_t malloced;
+	unsigned int arena_no;
 
 	LOCK(&contextslock);
 	ISC_LIST_UNLINK(contexts, ctx, link);
@@ -526,6 +583,8 @@ destroy(isc_mem_t *ctx) {
 
 	ctx->magic = 0;
 
+	arena_no = ctx->jemalloc_arena;
+
 	INSIST(ISC_LIST_EMPTY(ctx->pools));
 
 #if ISC_MEM_TRACKLINES
@@ -541,13 +600,14 @@ destroy(isc_mem_t *ctx) {
 				INSIST(!ctx->checkfree || dl->ptr == NULL);
 
 				ISC_LIST_UNLINK(ctx->debuglist[i], dl, link);
-				sdallocx(dl, sizeof(*dl), 0);
+				sdallocx(dl, sizeof(*dl), ctx->jemalloc_flags);
 				decrement_malloced(ctx, sizeof(*dl));
 			}
 		}
 
 		sdallocx(ctx->debuglist,
-			 (DEBUG_TABLE_COUNT * sizeof(debuglist_t)), 0);
+			 (DEBUG_TABLE_COUNT * sizeof(debuglist_t)),
+			 ctx->jemalloc_flags);
 		decrement_malloced(ctx,
 				   DEBUG_TABLE_COUNT * sizeof(debuglist_t));
 	}
@@ -578,7 +638,12 @@ destroy(isc_mem_t *ctx) {
 	if (ctx->checkfree) {
 		INSIST(malloced == 0);
 	}
-	sdallocx(ctx, sizeof(*ctx), MALLOCX_ALIGN(isc_os_cacheline()));
+	sdallocx(ctx, sizeof(*ctx),
+		 MALLOCX_ALIGN(isc_os_cacheline()) | ctx->jemalloc_flags);
+
+	if (arena_no != ISC_MEM_ILLEGAL_ARENA) {
+		RUNTIME_CHECK(mem_jemalloc_arena_destroy(arena_no) == true);
+	}
 }
 
 void
@@ -691,7 +756,7 @@ isc__mem_destroy(isc_mem_t **ctxp FLARG) {
 		}                                                      \
 	}
 
-static inline bool
+static bool
 hi_water(isc_mem_t *ctx) {
 	size_t inuse;
 	size_t maxinuse;
@@ -727,7 +792,7 @@ hi_water(isc_mem_t *ctx) {
 	return (true);
 }
 
-static inline bool
+static bool
 lo_water(isc_mem_t *ctx) {
 	size_t inuse;
 	size_t lowater = atomic_load_relaxed(&ctx->lo_water);
@@ -894,7 +959,7 @@ isc__mem_allocate(isc_mem_t *ctx, size_t size FLARG) {
 	ptr = mem_get(ctx, size, 0);
 
 	/* Recalculate the real allocated size */
-	size = sallocx(ptr, 0);
+	size = sallocx(ptr, ctx->jemalloc_flags);
 
 	mem_getstats(ctx, size);
 	ADD_TRACE(ctx, ptr, size, file, line);
@@ -947,7 +1012,7 @@ isc__mem_reallocate(isc_mem_t *ctx, void *old_ptr, size_t new_size FLARG) {
 	} else if (new_size == 0) {
 		isc__mem_free(ctx, old_ptr FLARG_PASS);
 	} else {
-		size_t old_size = sallocx(old_ptr, 0);
+		size_t old_size = sallocx(old_ptr, ctx->jemalloc_flags);
 
 		DELETE_TRACE(ctx, old_ptr, old_size, file, line);
 		mem_putstats(ctx, old_ptr, old_size);
@@ -955,7 +1020,7 @@ isc__mem_reallocate(isc_mem_t *ctx, void *old_ptr, size_t new_size FLARG) {
 		new_ptr = mem_realloc(ctx, old_ptr, old_size, new_size, 0);
 
 		/* Recalculate the real allocated size */
-		new_size = sallocx(new_ptr, 0);
+		new_size = sallocx(new_ptr, ctx->jemalloc_flags);
 
 		mem_getstats(ctx, new_size);
 		ADD_TRACE(ctx, new_ptr, new_size, file, line);
@@ -978,7 +1043,7 @@ isc__mem_free(isc_mem_t *ctx, void *ptr FLARG) {
 
 	REQUIRE(VALID_CONTEXT(ctx));
 
-	size = sallocx(ptr, 0);
+	size = sallocx(ptr, ctx->jemalloc_flags);
 
 	DELETE_TRACE(ctx, ptr, size, file, line);
 
@@ -1181,8 +1246,6 @@ isc__mempool_create(isc_mem_t *restrict mctx, const size_t element_size,
 	mpctx = isc_mem_get(mctx, sizeof(isc_mempool_t));
 
 	*mpctx = (isc_mempool_t){
-		.magic = MEMPOOL_MAGIC,
-		.mctx = mctx,
 		.size = size,
 		.freemax = 1,
 		.fillcount = 1,
@@ -1195,6 +1258,9 @@ isc__mempool_create(isc_mem_t *restrict mctx, const size_t element_size,
 	}
 #endif /* ISC_MEM_TRACKLINES */
 
+	isc_mem_attach(mctx, &mpctx->mctx);
+	mpctx->magic = MEMPOOL_MAGIC;
+
 	*mpctxp = (isc_mempool_t *)mpctx;
 
 	MCTXLOCK(mctx);
@@ -1233,10 +1299,7 @@ isc__mempool_destroy(isc_mempool_t **restrict mpctxp FLARG) {
 #endif
 
 	if (mpctx->allocated > 0) {
-		UNEXPECTED_ERROR(__FILE__, __LINE__,
-				 "isc_mempool_destroy(): mempool %s "
-				 "leaked memory",
-				 mpctx->name);
+		UNEXPECTED_ERROR("mempool %s leaked memory", mpctx->name);
 	}
 	REQUIRE(mpctx->allocated == 0);
 
@@ -1264,7 +1327,7 @@ isc__mempool_destroy(isc_mempool_t **restrict mpctxp FLARG) {
 
 	mpctx->magic = 0;
 
-	isc_mem_put(mpctx->mctx, mpctx, sizeof(isc_mempool_t));
+	isc_mem_putanddetach(&mpctx->mctx, mpctx, sizeof(isc_mempool_t));
 }
 
 void *
@@ -1404,7 +1467,8 @@ print_contexts(FILE *file) {
 	isc_mem_t *ctx;
 
 	for (ctx = ISC_LIST_HEAD(contexts); ctx != NULL;
-	     ctx = ISC_LIST_NEXT(ctx, link)) {
+	     ctx = ISC_LIST_NEXT(ctx, link))
+	{
 		fprintf(file, "context: %p (%s): %" PRIuFAST32 " references\n",
 			ctx, ctx->name[0] == 0 ? "<unknown>" : ctx->name,
 			isc_refcount_current(&ctx->references));
@@ -1414,7 +1478,7 @@ print_contexts(FILE *file) {
 }
 #endif
 
-static atomic_uintptr_t checkdestroyed = ATOMIC_VAR_INIT(0);
+static atomic_uintptr_t checkdestroyed = 0;
 
 void
 isc_mem_checkdestroyed(FILE *file) {
@@ -1436,8 +1500,7 @@ isc__mem_checkdestroyed(void) {
 			print_contexts(file);
 		}
 #endif /* if ISC_MEM_TRACKLINES */
-		INSIST(0);
-		ISC_UNREACHABLE();
+		UNREACHABLE();
 	}
 	UNLOCK(&contextslock);
 }
@@ -1561,7 +1624,8 @@ isc_mem_renderxml(void *writer0) {
 	LOCK(&contextslock);
 	lost = totallost;
 	for (ctx = ISC_LIST_HEAD(contexts); ctx != NULL;
-	     ctx = ISC_LIST_NEXT(ctx, link)) {
+	     ctx = ISC_LIST_NEXT(ctx, link))
+	{
 		xmlrc = xml_renderctx(ctx, &summary, writer);
 		if (xmlrc < 0) {
 			UNLOCK(&contextslock);
@@ -1703,7 +1767,8 @@ isc_mem_renderjson(void *memobj0) {
 	LOCK(&contextslock);
 	lost = totallost;
 	for (ctx = ISC_LIST_HEAD(contexts); ctx != NULL;
-	     ctx = ISC_LIST_NEXT(ctx, link)) {
+	     ctx = ISC_LIST_NEXT(ctx, link))
+	{
 		result = json_renderctx(ctx, &summary, ctxarray);
 		if (result != ISC_R_SUCCESS) {
 			UNLOCK(&contextslock);
@@ -1745,7 +1810,7 @@ error:
 
 void
 isc__mem_create(isc_mem_t **mctxp FLARG) {
-	mem_create(mctxp, isc_mem_defaultflags);
+	mem_create(mctxp, isc_mem_defaultflags, 0);
 #if ISC_MEM_TRACKLINES
 	if ((isc_mem_debugging & ISC_MEM_DEBUGTRACE) != 0) {
 		fprintf(stderr, "create mctx %p file %s line %u\n", *mctxp,
@@ -1754,6 +1819,83 @@ isc__mem_create(isc_mem_t **mctxp FLARG) {
 #endif /* ISC_MEM_TRACKLINES */
 }
 
+void
+isc__mem_create_arena(isc_mem_t **mctxp FLARG) {
+	unsigned int arena_no = ISC_MEM_ILLEGAL_ARENA;
+
+	RUNTIME_CHECK(mem_jemalloc_arena_create(&arena_no));
+
+	/*
+	 * We use MALLOCX_TCACHE_NONE to bypass the tcache and route
+	 * allocations directly to the arena. That is a recommendation
+	 * from jemalloc developers:
+	 *
+	 * https://github.com/jemalloc/jemalloc/issues/2483#issuecomment-1698173849
+	 */
+	mem_create(mctxp, isc_mem_defaultflags,
+		   arena_no == ISC_MEM_ILLEGAL_ARENA
+			   ? 0
+			   : MALLOCX_ARENA(arena_no) | MALLOCX_TCACHE_NONE);
+	(*mctxp)->jemalloc_arena = arena_no;
+#if ISC_MEM_TRACKLINES
+	if ((isc_mem_debugging & ISC_MEM_DEBUGTRACE) != 0) {
+		fprintf(stderr,
+			"create mctx %p file %s line %u for jemalloc arena "
+			"%u\n",
+			*mctxp, file, line, arena_no);
+	}
+#endif /* ISC_MEM_TRACKLINES */
+}
+
+#if defined(JEMALLOC_API_SUPPORTED) && JEMALLOC_VERSION_MAJOR >= 4
+static bool
+jemalloc_set_ssize_value(const char *valname, ssize_t newval) {
+	int ret;
+
+	ret = mallctl(valname, NULL, NULL, &newval, sizeof(newval));
+	return (ret == 0);
+}
+#endif /* defined(JEMALLOC_API_SUPPORTED) && JEMALLOC_VERSION_MAJOR >= 4 */
+
+static isc_result_t
+mem_set_arena_ssize_value(isc_mem_t *mctx, const char *arena_valname,
+			  const ssize_t newval) {
+	REQUIRE(VALID_CONTEXT(mctx));
+#if defined(JEMALLOC_API_SUPPORTED) && JEMALLOC_VERSION_MAJOR >= 4
+	bool ret;
+	char buf[256] = { 0 };
+
+	if (mctx->jemalloc_arena == ISC_MEM_ILLEGAL_ARENA) {
+		return (ISC_R_UNEXPECTED);
+	}
+
+	(void)snprintf(buf, sizeof(buf), "arena.%u.%s", mctx->jemalloc_arena,
+		       arena_valname);
+
+	ret = jemalloc_set_ssize_value(buf, newval);
+
+	if (!ret) {
+		return (ISC_R_FAILURE);
+	}
+
+	return (ISC_R_SUCCESS);
+#else
+	UNUSED(arena_valname);
+	UNUSED(newval);
+	return (ISC_R_NOTIMPLEMENTED);
+#endif /* defined(JEMALLOC_API_SUPPORTED) && JEMALLOC_VERSION_MAJOR >= 4 */
+}
+
+isc_result_t
+isc_mem_arena_set_muzzy_decay_ms(isc_mem_t *mctx, const ssize_t decay_ms) {
+	return (mem_set_arena_ssize_value(mctx, "muzzy_decay_ms", decay_ms));
+}
+
+isc_result_t
+isc_mem_arena_set_dirty_decay_ms(isc_mem_t *mctx, const ssize_t decay_ms) {
+	return (mem_set_arena_ssize_value(mctx, "dirty_decay_ms", decay_ms));
+}
+
 void
 isc__mem_printactive(isc_mem_t *ctx, FILE *file) {
 #if ISC_MEM_TRACKLINES
@@ -1766,3 +1908,14 @@ isc__mem_printactive(isc_mem_t *ctx, FILE *file) {
 	UNUSED(file);
 #endif /* if ISC_MEM_TRACKLINES */
 }
+
+void *
+isc__mem_alloc_noctx(size_t size) {
+	return mallocx(size, 0);
+}
+
+void
+isc__mem_free_noctx(void *ptr, size_t size) {
+	ADJUST_ZERO_ALLOCATION_SIZE(size);
+	sdallocx(ptr, size, 0);
+}
