@@ -1,9 +1,11 @@
 /*
  * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
+ * SPDX-License-Identifier: MPL-2.0
+ *
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
+ * file, you can obtain one at https://mozilla.org/MPL/2.0/.
  *
  * See the COPYRIGHT file distributed with this work for additional
  * information regarding copyright ownership.
@@ -13,7 +15,6 @@
 #include <uv.h>
 
 #include <isc/result.h>
-#include <isc/strerr.h>
 #include <isc/string.h>
 #include <isc/util.h>
 
@@ -27,8 +28,10 @@
  */
 isc_result_t
 isc___nm_uverr2result(int uverr, bool dolog, const char *file,
-		      unsigned int line) {
+		      unsigned int line, const char *func) {
 	switch (uverr) {
+	case 0:
+		return (ISC_R_SUCCESS);
 	case UV_ENOTDIR:
 	case UV_ELOOP:
 	case UV_EINVAL: /* XXX sometimes this is not for files */
@@ -79,12 +82,23 @@ isc___nm_uverr2result(int uverr, bool dolog, const char *file,
 		return (ISC_R_ADDRNOTAVAIL);
 	case UV_ECONNREFUSED:
 		return (ISC_R_CONNREFUSED);
+	case UV_ECANCELED:
+		return (ISC_R_CANCELED);
+	case UV_EOF:
+		return (ISC_R_EOF);
+	case UV_EMSGSIZE:
+		return (ISC_R_MAXSIZE);
+	case UV_ENOTSUP:
+		return (ISC_R_FAMILYNOSUPPORT);
+	case UV_ENOPROTOOPT:
+	case UV_EPROTONOSUPPORT:
+		return (ISC_R_INVALIDPROTO);
 	default:
 		if (dolog) {
-			UNEXPECTED_ERROR(file, line,
-					 "unable to convert libuv "
-					 "error code to isc_result: %d: %s",
-					 uverr, uv_strerror(uverr));
+			UNEXPECTED_ERROR("unable to convert libuv error code "
+					 "in %s (%s:%d) to isc_result: %d: %s",
+					 func, file, line, uverr,
+					 uv_strerror(uverr));
 		}
 		return (ISC_R_UNEXPECTED);
 	}
